(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 10.4' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       158,          7]
NotebookDataLength[     92591,       2547]
NotebookOptionsPosition[     85946,       2322]
NotebookOutlinePosition[     86300,       2338]
CellTagsIndexPosition[     86257,       2335]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell[TextData[{
 "Theoretical analysis of paradoxical effects in inhibitory-stabilised \
networks (ISNs)\n",
 StyleBox["We analyse here a simple network (Wilson-Cowan style) with equal \
numbers of excitatory and inhibitory neurons. See the accompanying manuscript \
for details (Sadeh et al. 2017, ", "Text"],
 StyleBox["J Neurosci.", "Text",
  FontSlant->"Italic"],
 StyleBox[").", "Text"]
}], "Title",
 CellChangeTimes->{{3.670588122555388*^9, 3.670588133434986*^9}, {
  3.67064233483815*^9, 3.670642376743143*^9}, {3.718448130124095*^9, 
  3.71844816923273*^9}}],

Cell[CellGroupData[{

Cell[TextData[{
 "Network definition\n",
 StyleBox["NN", "Input"],
 StyleBox[" defines the number of excitatory, and equal number of inhibitory \
neurons in the network. The network therefore contains ", "Text"],
 StyleBox["2NN", "Input"],
 StyleBox[" neurons.", "Text"]
}], "Section",
 CellChangeTimes->{{3.670588138481484*^9, 3.670588155192894*^9}, {
  3.670642380087284*^9, 3.670642434909782*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"NN", ":=", "8"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"a", ":=", 
   RowBox[{"Flatten", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"Indexed", "[", 
         RowBox[{"x", ",", "i"}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "1", ",", "NN"}], "}"}]}], "]"}], ",", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"Indexed", "[", 
         RowBox[{"y", ",", "i"}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "1", ",", "NN"}], "}"}]}], "]"}]}], "}"}], "]"}]}],
   ";", 
  RowBox[{"T", ":=", 
   RowBox[{"Table", "[", 
    RowBox[{"\[Tau]", ",", 
     RowBox[{"{", 
      RowBox[{"2", "NN"}], "}"}]}], "]"}]}], ";", 
  RowBox[{"inp", ":=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"Indexed", "[", 
      RowBox[{"\[Iota]", ",", "i"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", 
       RowBox[{"2", "NN"}]}], "}"}]}], "]"}]}], ";", 
  RowBox[{"W", ":=", 
   RowBox[{"Transpose", "[", 
    RowBox[{"ArrayReshape", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"we", "/", "NN"}], ",", 
          RowBox[{"{", 
           RowBox[{"2", "NN"}], "}"}], ",", 
          RowBox[{"{", "NN", "}"}]}], "]"}], ",", " ", 
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "wi"}], "/", "NN"}], ",", 
          RowBox[{"{", 
           RowBox[{"2", "NN"}], "}"}], ",", 
          RowBox[{"{", "NN", "}"}]}], "]"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"2", "NN"}], ",", 
        RowBox[{"2", "NN"}]}], "}"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"J", ":=", 
  RowBox[{"ArrayReshape", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"W", "-", 
       RowBox[{"IdentityMatrix", "[", 
        RowBox[{"2", "NN"}], "]"}]}], ")"}], "/", 
     RowBox[{"Table", "[", 
      RowBox[{"T", ",", 
       RowBox[{"{", 
        RowBox[{"2", "NN"}], "}"}]}], "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"2", "NN"}], ",", 
      RowBox[{"2", "NN"}]}], "}"}]}], "]"}]}]}], "Input",
 CellChangeTimes->{{3.670042981240307*^9, 3.670043131475662*^9}, {
   3.670043168377324*^9, 3.670043218240018*^9}, {3.670044973871442*^9, 
   3.6700450005154133`*^9}, {3.670045140336553*^9, 3.6700451450871553`*^9}, {
   3.670045299500021*^9, 3.670045361056547*^9}, {3.670049530857359*^9, 
   3.670049541374296*^9}, {3.670050498086627*^9, 3.670050531411861*^9}, {
   3.6700505993365383`*^9, 3.670050658221753*^9}, 3.670050705850541*^9, {
   3.670050765540203*^9, 3.670050899021755*^9}, {3.670050956900794*^9, 
   3.6700509589713078`*^9}, {3.670051270876419*^9, 3.6700513194939833`*^9}, {
   3.670051388637054*^9, 3.670051525041828*^9}, {3.670051573665234*^9, 
   3.670051690341251*^9}, {3.6700518035235023`*^9, 3.67005180692278*^9}, {
   3.670051921166394*^9, 3.67005193402103*^9}, {3.67005203315545*^9, 
   3.670052039988351*^9}, {3.670052346590644*^9, 3.670052353932054*^9}, {
   3.670052432331183*^9, 3.670052432456785*^9}, {3.670052830300229*^9, 
   3.670052844871526*^9}, {3.6700528864538317`*^9, 3.670052907065353*^9}, {
   3.670052994945013*^9, 3.6700529950323677`*^9}, 3.6700530451776667`*^9, {
   3.67005307712072*^9, 3.6700530795024977`*^9}, {3.670053237521813*^9, 
   3.670053242935686*^9}, {3.670053847825061*^9, 3.670053847925873*^9}, {
   3.670054389207089*^9, 3.67005438925956*^9}, 3.6700588210376797`*^9, {
   3.6700590523881607`*^9, 3.6700590527541237`*^9}, 3.6700593306661043`*^9, {
   3.670059458660577*^9, 3.6700594593774137`*^9}, {3.670124198169723*^9, 
   3.67012419935209*^9}, {3.670124661616107*^9, 3.6701246621758432`*^9}, {
   3.670124951277515*^9, 3.6701249514693327`*^9}, {3.670127461301209*^9, 
   3.6701275194411507`*^9}, 3.67012766387659*^9, 3.670235513536289*^9, {
   3.6705836668237963`*^9, 3.670583667070532*^9}, {3.695027897896269*^9, 
   3.695027928388689*^9}}],

Cell[BoxData[
 RowBox[{"System", ":=", 
  RowBox[{"MapThread", "[", 
   RowBox[{
    RowBox[{"Function", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"x1", ",", "x2"}], "}"}], ",", 
      RowBox[{"x1", "\[Equal]", "x2"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"T", " ", 
        RowBox[{"Map", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"#", "'"}], "&"}], ",", 
          RowBox[{"Flatten", "[", "a", "]"}]}], "]"}]}], "+", "a"}], ",", 
      RowBox[{
       RowBox[{"Dot", "[", 
        RowBox[{"W", ",", "a"}], "]"}], "+", "inp"}]}], "}"}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.6700434766114798`*^9, 3.670043510476014*^9}, {
   3.670043554795559*^9, 3.670043593145031*^9}, {3.670043639623253*^9, 
   3.670043726525116*^9}, {3.6700438809892807`*^9, 3.670043883964903*^9}, {
   3.6700440596228313`*^9, 3.670044062846081*^9}, {3.6700440933506117`*^9, 
   3.6700441784032497`*^9}, {3.6700513096248198`*^9, 3.670051310192539*^9}, {
   3.670051776028146*^9, 3.6700517804507313`*^9}, {3.670051951117432*^9, 
   3.670051951235888*^9}, 3.670051996055444*^9, {3.670052049987441*^9, 
   3.6700520677837563`*^9}, {3.6700521168338842`*^9, 3.6700521356387167`*^9}}],

Cell["\<\
Examine the eigenspectrum of the system Jacobian\
\>", "Text",
 CellChangeTimes->{{3.718447430568858*^9, 3.718447442032187*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"J", "//", "MatrixForm"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"eigVals", ",", "eigVecs"}], "}"}], "=", 
   RowBox[{"Eigensystem", "[", "J", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"eigVals", "\[LeftDoubleBracket]", 
  RowBox[{"2", "NN"}], "\[RightDoubleBracket]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"eigVals", "\[LeftDoubleBracket]", 
   RowBox[{"2", "NN"}], "\[RightDoubleBracket]"}], "/.", 
  RowBox[{"wi", "\[Rule]", "0"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Tr", "[", "J", "]"}], "//", "FullSimplify"}]}], "Input",
 CellChangeTimes->{{3.670044935103554*^9, 3.670044938567543*^9}, {
   3.670045015331994*^9, 3.670045015602631*^9}, {3.670045161446539*^9, 
   3.670045186111347*^9}, 3.6700528520544653`*^9, {3.670058809584548*^9, 
   3.6700588168941517`*^9}, {3.670059363010219*^9, 3.670059370296957*^9}, {
   3.670124202492731*^9, 3.670124206681445*^9}, {3.675834948113929*^9, 
   3.675834981611463*^9}, {3.675835459239973*^9, 3.675835464749646*^9}, 
   3.67583562072281*^9, {3.6758358967183037`*^9, 3.6758359071725187`*^9}, {
   3.67729194461034*^9, 3.677292094471945*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"-", "1"}], "+", "we", "-", "wi"}], "\[Tau]"]], "Output",
 CellChangeTimes->{{3.67729198972161*^9, 3.6772920447172203`*^9}, 
   3.67729209660567*^9, 3.677292978789822*^9, 3.677293014720964*^9, 
   3.6773016454952707`*^9, {3.695027934127719*^9, 3.695027939877249*^9}, 
   3.7184475434024343`*^9}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"-", "1"}], "+", "we"}], "\[Tau]"]], "Output",
 CellChangeTimes->{{3.67729198972161*^9, 3.6772920447172203`*^9}, 
   3.67729209660567*^9, 3.677292978789822*^9, 3.677293014720964*^9, 
   3.6773016454952707`*^9, {3.695027934127719*^9, 3.695027939877249*^9}, 
   3.718447543406049*^9}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"-", "16"}], "+", "we", "-", "wi"}], "\[Tau]"]], "Output",
 CellChangeTimes->{{3.67729198972161*^9, 3.6772920447172203`*^9}, 
   3.67729209660567*^9, 3.677292978789822*^9, 3.677293014720964*^9, 
   3.6773016454952707`*^9, {3.695027934127719*^9, 3.695027939877249*^9}, 
   3.718447543526435*^9}]
}, Open  ]],

Cell["Make a replacement rule for the trivial eigenvalue", "Text",
 CellChangeTimes->{{3.67729211009179*^9, 3.677292136235239*^9}}],

Cell[BoxData[
 RowBox[{"eigValRep", ":=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"Evaluate", "[", 
      RowBox[{"\[Tau]", " ", 
       RowBox[{"eigVals", "\[LeftDoubleBracket]", 
        RowBox[{"2", "NN"}], "\[RightDoubleBracket]"}]}], "]"}], "\[Rule]", 
     SubscriptBox["\[Lambda]", "1"]}], ",", 
    RowBox[{
     RowBox[{"wi", "-", "we", "+", "1"}], "\[Rule]", 
     RowBox[{"-", 
      SubscriptBox["\[Lambda]", "1"]}]}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.677292908189067*^9, 3.677292957678145*^9}, {
  3.677293022792183*^9, 3.67729302439624*^9}, {3.677301733986661*^9, 
  3.677301752106144*^9}, {3.6773018787215843`*^9, 3.6773018951291533`*^9}}],

Cell[CellGroupData[{

Cell[BoxData["eigValRep"], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"-", "1"}], "+", "we", "-", "wi"}], "\[Rule]", 
    SubscriptBox["\[Lambda]", "1"]}], ",", 
   RowBox[{
    RowBox[{"1", "-", "we", "+", "wi"}], "\[Rule]", 
    RowBox[{"-", 
     SubscriptBox["\[Lambda]", "1"]}]}]}], "}"}]], "Output",
 CellChangeTimes->{{3.677292961833754*^9, 3.677292978866973*^9}, {
   3.677293014814355*^9, 3.677293027972498*^9}, 3.67730164555756*^9, 
   3.677301898128293*^9, {3.695027934306672*^9, 3.695027939957593*^9}, 
   3.7184475436047697`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "Conditions for BIBO stability\n",
 StyleBox["The network is BIBO stable if all eigenvalues of ", "Text"],
 StyleBox["J", "Input",
  FontWeight->"Bold"],
 StyleBox[" have a real part < 0", "Text"]
}], "Section",
 CellChangeTimes->{{3.670588159320301*^9, 3.670588164720345*^9}, {
  3.71844747161513*^9, 3.718447527573428*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StabilityCond", "=", 
  RowBox[{
   RowBox[{"Reduce", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"eigVals", "\[LeftDoubleBracket]", 
         RowBox[{"2", "NN"}], "\[RightDoubleBracket]"}], "<", "0"}], ",", 
       RowBox[{
        RowBox[{"Tr", "[", "J", "]"}], "<", "0"}], ",", 
       RowBox[{"\[Tau]", ">", "0"}], ",", 
       RowBox[{"we", "\[GreaterEqual]", "0"}], ",", 
       RowBox[{"wi", "\[GreaterEqual]", "0"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"we", ",", "wi"}], "}"}], ",", "Reals"}], "]"}], "//", 
   "FullSimplify"}]}]], "Input",
 CellChangeTimes->{{3.670045203365904*^9, 3.670045284059214*^9}, {
   3.6700454187924767`*^9, 3.670045425718577*^9}, {3.670046023495009*^9, 
   3.670046025717367*^9}, {3.670051719820855*^9, 3.670051719932746*^9}, 
   3.670059388740717*^9, 3.67012752598527*^9, {3.6758360028753633`*^9, 
   3.6758360071215963`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Tau]", ">", "0"}], "&&", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"0", "\[LessEqual]", "we", "<", "1"}], "&&", 
      RowBox[{"wi", "\[GreaterEqual]", "0"}]}], ")"}], "||", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"we", "\[GreaterEqual]", "1"}], "&&", 
      RowBox[{
       RowBox[{"1", "+", "wi"}], ">", "we"}]}], ")"}]}], ")"}]}]], "Output",
 CellChangeTimes->{{3.670045208119286*^9, 3.67004528428424*^9}, {
   3.670045337919393*^9, 3.670045363462837*^9}, {3.6700454260109367`*^9, 
   3.670045443858457*^9}, {3.6700460262460127`*^9, 3.6700460386464767`*^9}, 
   3.67004955072115*^9, 3.6700497041532507`*^9, 3.670050250258623*^9, 
   3.670050306356823*^9, 3.670050358920743*^9, 3.6700503995882587`*^9, 
   3.670050439146399*^9, 3.6700508781796713`*^9, 3.6700517216278067`*^9, 
   3.670051821124836*^9, 3.670052147219406*^9, {3.670052344317191*^9, 
   3.670052354768525*^9}, 3.6700524334184237`*^9, 3.6700525398502817`*^9, {
   3.67005261822507*^9, 3.67005262944188*^9}, 3.670052716035725*^9, {
   3.670052812151204*^9, 3.670052907871249*^9}, 3.670052996459753*^9, 
   3.6700530458430443`*^9, 3.670053081113637*^9, {3.670053240150979*^9, 
   3.6700532435762377`*^9}, 3.670053849125678*^9, 3.6700543899488087`*^9, 
   3.670059053335804*^9, 3.6700593331039457`*^9, 3.670059391729542*^9, 
   3.6700594636218224`*^9, 3.670083304699616*^9, 3.670124200206935*^9, 
   3.670124662762917*^9, {3.670124948233067*^9, 3.670124952465085*^9}, 
   3.670127672423357*^9, 3.670210521536729*^9, 3.6702355143860598`*^9, 
   3.67058366841306*^9, 3.67058370920343*^9, 3.6705862388020773`*^9, 
   3.67058850319523*^9, 3.675835853601416*^9, 3.675836008791037*^9, 
   3.676007752980867*^9, 3.677301645600045*^9, 3.695027940047104*^9, 
   3.7184475437089043`*^9}]
}, Open  ]],

Cell[TextData[{
 "Examine the constraints for network stability in the absence of inhibition \
(i.e. ",
 StyleBox["wi\[Rule]0", "Input"],
 ")"
}], "Text",
 CellChangeTimes->{{3.71844757069154*^9, 3.7184475982169*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"StabilityCond", "/.", 
   RowBox[{"wi", "\[Rule]", "0"}]}], "//", "FullSimplify"}]], "Input",
 CellChangeTimes->{{3.67008408497027*^9, 3.6700840872170353`*^9}, {
  3.6758358761194677`*^9, 3.6758358787253*^9}, {3.675836016817164*^9, 
  3.675836022787471*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Tau]", ">", "0"}], "&&", 
  RowBox[{"0", "\[LessEqual]", "we", "<", "1"}]}]], "Output",
 CellChangeTimes->{
  3.670124200266736*^9, 3.670124662787315*^9, {3.670124948270557*^9, 
   3.670124952489292*^9}, 3.670127672462782*^9, 3.6702105215809183`*^9, 
   3.670235514416151*^9, 3.6705836684545918`*^9, 3.6705837092534723`*^9, 
   3.6705862388513517`*^9, 3.670588503242332*^9, {3.67583585391588*^9, 
   3.675835878961166*^9}, {3.6758360106035748`*^9, 3.675836023029023*^9}, 
   3.6760077530244503`*^9, 3.677301645643447*^9, 3.6950279400918503`*^9, 
   3.718447543757201*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Reduce", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Tr", "[", "J", "]"}], "<", "0"}], ",", 
     RowBox[{"\[Tau]", ">", "0"}]}], "}"}], ",", 
   RowBox[{"{", "we", "}"}], ",", "Reals"}], "]"}]], "Input",
 CellChangeTimes->{{3.6700594101592703`*^9, 3.670059439826023*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Tau]", ">", "0"}], "&&", 
  RowBox[{"we", "<", 
   RowBox[{"16", "+", "wi"}]}]}]], "Output",
 CellChangeTimes->{{3.67005941989855*^9, 3.6700594636614513`*^9}, 
   3.670083304747994*^9, 3.6701242003214817`*^9, 3.670124662826661*^9, {
   3.670124948310493*^9, 3.67012495252254*^9}, 3.670127672506235*^9, 
   3.670210521632635*^9, 3.670235514474059*^9, 3.670583668493412*^9, 
   3.670583709316739*^9, 3.67058623890134*^9, 3.6705885032915087`*^9, 
   3.6758358539933157`*^9, 3.6760077530709*^9, 3.677301645684849*^9, 
   3.6950279401604767`*^9, 3.7184475438066187`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "Fixed-point response of the network\n",
 StyleBox["We obtain the fixed-point response by solving for the point where \
", "Text"],
 StyleBox["a\[CloseCurlyQuote][t]==0", "Input"],
 StyleBox[".", "Text"]
}], "Section",
 CellChangeTimes->{{3.670588170143775*^9, 3.6705881790234747`*^9}, {
  3.670642439933969*^9, 3.670642468901318*^9}}],

Cell[BoxData[
 RowBox[{"FPSys", ":=", 
  RowBox[{"a", "/.", 
   RowBox[{
    RowBox[{"Solve", "[", 
     RowBox[{
      RowBox[{"System", "/.", 
       RowBox[{"\[Tau]", "\[Rule]", "0"}]}], ",", "a"}], "]"}], 
    "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}]}]], "Input",
 CellChangeTimes->{{3.670043681533535*^9, 3.670043693742571*^9}, {
  3.670043896555539*^9, 3.6700439464589376`*^9}, {3.670044221010367*^9, 
  3.670044279640583*^9}, {3.6700443163121653`*^9, 3.670044336623148*^9}, {
  3.670044371282672*^9, 3.670044371991243*^9}, {3.6700445577275877`*^9, 
  3.670044562060635*^9}, {3.6700499071067247`*^9, 3.670049910954501*^9}, {
  3.670049957343495*^9, 3.670049964805097*^9}, {3.6700518424931173`*^9, 
  3.6700518437884083`*^9}, {3.670052155070806*^9, 3.67005218357913*^9}, {
  3.670642488072605*^9, 3.670642489861753*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FPSys", "/.", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"Indexed", "[", 
        RowBox[{"\[Iota]", ",", "i"}], "]"}], "\[Rule]", 
       SubscriptBox["\[Iota]", "inh"]}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", 
        RowBox[{"NN", "+", "1"}], ",", 
        RowBox[{"2", "NN"}]}], "}"}]}], "]"}]}], "//", "FullSimplify"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.670044546904202*^9, 3.670044547669837*^9}, {
   3.67004458272288*^9, 3.67004461987589*^9}, {3.670049883196067*^9, 
   3.670049893890513*^9}, {3.670052201329289*^9, 3.6700522452246428`*^9}, {
   3.6700528016828823`*^9, 3.670052808905704*^9}, 3.670052872636561*^9, {
   3.67012753773916*^9, 3.670127540512578*^9}, {3.670588483627324*^9, 
   3.6705884860160837`*^9}, 3.670642493653611*^9, {3.6706425248212013`*^9, 
   3.670642526774563*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"FPSys", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], "//",
   "FullSimplify"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"FPSys", "\[LeftDoubleBracket]", 
   RowBox[{"2", "NN"}], "\[RightDoubleBracket]"}], "//", 
  "FullSimplify"}]}], "Input",
 CellChangeTimes->{{3.670588488735097*^9, 3.6705884983773746`*^9}, {
  3.675837525413602*^9, 3.675837544957972*^9}, {3.677298595811635*^9, 
  3.6772985967390413`*^9}, {3.6772987460862083`*^9, 3.677298757859968*^9}, {
  3.677298846791913*^9, 3.677298871279142*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox["1", 
   RowBox[{"8", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", "we", "-", "wi"}], ")"}]}]], 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"7", " ", "we"}], "-", 
       RowBox[{"8", " ", 
        RowBox[{"(", 
         RowBox[{"1", "+", "wi"}], ")"}]}]}], ")"}], " ", 
     SubscriptBox["\[Iota]", "1"]}], "-", 
    RowBox[{"we", " ", 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[Iota]", "2"], "+", 
       SubscriptBox["\[Iota]", "3"], "+", 
       SubscriptBox["\[Iota]", "4"], "+", 
       SubscriptBox["\[Iota]", "5"], "+", 
       SubscriptBox["\[Iota]", "6"], "+", 
       SubscriptBox["\[Iota]", "7"], "+", 
       SubscriptBox["\[Iota]", "8"]}], ")"}]}], "+", 
    RowBox[{"wi", " ", 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[Iota]", "9"], "+", 
       SubscriptBox["\[Iota]", "10"], "+", 
       SubscriptBox["\[Iota]", "11"], "+", 
       SubscriptBox["\[Iota]", "12"], "+", 
       SubscriptBox["\[Iota]", "13"], "+", 
       SubscriptBox["\[Iota]", "14"], "+", 
       SubscriptBox["\[Iota]", "15"], "+", 
       SubscriptBox["\[Iota]", "16"]}], ")"}]}]}], ")"}]}]], "Output",
 CellChangeTimes->{{3.670588494303116*^9, 3.670588503561919*^9}, {
   3.675837526762488*^9, 3.67583754545009*^9}, 3.676007754490295*^9, 
   3.677293348338519*^9, 3.677298597599937*^9, 3.677298758389327*^9, {
   3.6772988489473553`*^9, 3.677298871633202*^9}, 3.67730164688334*^9, 
   3.695027941116075*^9, 3.7184475448254642`*^9}],

Cell[BoxData[
 RowBox[{
  FractionBox["1", 
   RowBox[{"8", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", "we", "-", "wi"}], ")"}]}]], 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"-", "we"}], " ", 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[Iota]", "1"], "+", 
       SubscriptBox["\[Iota]", "2"], "+", 
       SubscriptBox["\[Iota]", "3"], "+", 
       SubscriptBox["\[Iota]", "4"], "+", 
       SubscriptBox["\[Iota]", "5"], "+", 
       SubscriptBox["\[Iota]", "6"], "+", 
       SubscriptBox["\[Iota]", "7"], "+", 
       SubscriptBox["\[Iota]", "8"]}], ")"}]}], "+", 
    RowBox[{"wi", " ", 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[Iota]", "9"], "+", 
       SubscriptBox["\[Iota]", "10"], "+", 
       SubscriptBox["\[Iota]", "11"], "+", 
       SubscriptBox["\[Iota]", "12"], "+", 
       SubscriptBox["\[Iota]", "13"], "+", 
       SubscriptBox["\[Iota]", "14"], "+", 
       SubscriptBox["\[Iota]", "15"]}], ")"}]}], "-", 
    RowBox[{"8", " ", 
     SubscriptBox["\[Iota]", "16"]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"8", " ", "we"}], "-", 
       RowBox[{"7", " ", "wi"}]}], ")"}], " ", 
     SubscriptBox["\[Iota]", "16"]}]}], ")"}]}]], "Output",
 CellChangeTimes->{{3.670588494303116*^9, 3.670588503561919*^9}, {
   3.675837526762488*^9, 3.67583754545009*^9}, 3.676007754490295*^9, 
   3.677293348338519*^9, 3.677298597599937*^9, 3.677298758389327*^9, {
   3.6772988489473553`*^9, 3.677298871633202*^9}, 3.67730164688334*^9, 
   3.695027941116075*^9, 3.718447545052474*^9}]
}, Open  ]],

Cell[TextData[{
 "General form of fixed-point response (Note that ",
 StyleBox["j", "Input",
  FontSlant->"Italic"],
 " has different ranges for ",
 Cell[BoxData[
  FormBox[
   StyleBox[
    SubscriptBox["x", "j"], "Input"], TraditionalForm]], "input",
  FormatType->"TraditionalForm"],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["y", "j"], TraditionalForm]], "Input",
  FormatType->"TraditionalForm"],
 ")"
}], "Text",
 CellChangeTimes->{{3.677295276419312*^9, 3.677295286505485*^9}, {
  3.695027956716382*^9, 3.6950280175514593`*^9}, {3.7184476244325733`*^9, 
  3.7184476518429527`*^9}}],

Cell[BoxData[
 RowBox[{
  SubscriptBox[
   OverscriptBox["x", "_"], "j_"], ":=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"wi", " ", 
      SubscriptBox["\[CapitalSigma]", "i"]}], "-", 
     RowBox[{"we", " ", 
      SubscriptBox["\[CapitalSigma]", "e"]}], "+", 
     RowBox[{"NN", " ", 
      SubscriptBox["\[Lambda]", "1"], 
      SubscriptBox["\[Iota]", "j"]}]}], ")"}], "/", 
   RowBox[{"(", 
    RowBox[{"NN", " ", 
     SubscriptBox["\[Lambda]", "1"]}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.677298781420755*^9, 3.6772988261656017`*^9}, {
  3.67729908267307*^9, 3.6772990960294743`*^9}}],

Cell[BoxData[
 RowBox[{
  SubscriptBox[
   OverscriptBox["y", "_"], "j_"], ":=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"wi", " ", 
      SubscriptBox["\[CapitalSigma]", "i"]}], "-", 
     RowBox[{"we", " ", 
      SubscriptBox["\[CapitalSigma]", "e"]}], "+", 
     RowBox[{"NN", " ", 
      SubscriptBox["\[Lambda]", "1"], 
      SubscriptBox["\[Iota]", "j"]}]}], ")"}], "/", 
   RowBox[{"(", 
    RowBox[{"NN", " ", 
     SubscriptBox["\[Lambda]", "1"]}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.677295703141005*^9, 3.677295820920824*^9}, {
   3.6772958660739117`*^9, 3.67729591569656*^9}, {3.677295977527502*^9, 
   3.677295980897029*^9}, {3.677296079604677*^9, 3.677296090636866*^9}, {
   3.677296297901268*^9, 3.677296464280025*^9}, {3.677296854128483*^9, 
   3.677296854881813*^9}, {3.6772969080406713`*^9, 3.67729704188936*^9}, {
   3.677297087002679*^9, 3.6772971129579144`*^9}, {3.6772971515498257`*^9, 
   3.67729722308261*^9}, {3.6772985009927397`*^9, 3.677298502894321*^9}, {
   3.6772985629222107`*^9, 3.677298580831995*^9}, {3.677298661358596*^9, 
   3.6772986639809217`*^9}, {3.677299005691608*^9, 3.677299014577886*^9}, 
   3.6772991288709106`*^9}],

Cell[CellGroupData[{

Cell[BoxData[{
 SubscriptBox[
  OverscriptBox["x", "_"], "1"], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      SubscriptBox[
       OverscriptBox["x", "_"], "1"], "-", 
      RowBox[{
      "FPSys", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}], 
     "\[Equal]", "0"}], "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[CapitalSigma]", "e"], "\[Rule]", 
       RowBox[{"Plus", "@@", 
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"Indexed", "[", 
           RowBox[{"\[Iota]", ",", "i"}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"i", ",", "1", ",", "NN"}], "}"}]}], "]"}]}]}], ",", 
      RowBox[{
       SubscriptBox["\[CapitalSigma]", "i"], "\[Rule]", 
       RowBox[{"Plus", "@@", 
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"Indexed", "[", 
           RowBox[{"\[Iota]", ",", "i"}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"i", ",", 
            RowBox[{"NN", "+", "1"}], ",", 
            RowBox[{"2", "NN"}]}], "}"}]}], "]"}]}]}], ",", 
      RowBox[{"Reverse", "[", 
       RowBox[{
       "eigValRep", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
       "]"}]}], "}"}]}], "//", "Expand"}], "//", "FullSimplify"}]}], "Input",
 CellChangeTimes->{{3.677299149589747*^9, 3.677299190068255*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"8", " ", 
    SubscriptBox["\[Iota]", "1"], " ", 
    SubscriptBox["\[Lambda]", "1"]}], "-", 
   RowBox[{"we", " ", 
    SubscriptBox["\[CapitalSigma]", "e"]}], "+", 
   RowBox[{"wi", " ", 
    SubscriptBox["\[CapitalSigma]", "i"]}]}], 
  RowBox[{"8", " ", 
   SubscriptBox["\[Lambda]", "1"]}]]], "Output",
 CellChangeTimes->{{3.677299168055717*^9, 3.677299199284009*^9}, 
   3.6773016470037622`*^9, 3.695027941445916*^9, 3.718447545144062*^9}],

Cell[BoxData[
 RowBox[{
  SubscriptBox["\[Iota]", "1"], "\[Equal]", 
  SubscriptBox["\[Iota]", "1"]}]], "Output",
 CellChangeTimes->{{3.677299168055717*^9, 3.677299199284009*^9}, 
   3.6773016470037622`*^9, 3.695027941445916*^9, 3.718447545198948*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 SubscriptBox[
  OverscriptBox["y", "_"], 
  RowBox[{"NN", "+", "1"}]], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      SubscriptBox[
       OverscriptBox["y", "_"], 
       RowBox[{"NN", "+", "1"}]], "-", 
      RowBox[{"FPSys", "\[LeftDoubleBracket]", 
       RowBox[{"NN", "+", "1"}], "\[RightDoubleBracket]"}]}], "\[Equal]", 
     "0"}], "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[CapitalSigma]", "e"], "\[Rule]", 
       RowBox[{"Plus", "@@", 
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"Indexed", "[", 
           RowBox[{"\[Iota]", ",", "i"}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"i", ",", "1", ",", "NN"}], "}"}]}], "]"}]}]}], ",", 
      RowBox[{
       SubscriptBox["\[CapitalSigma]", "i"], "\[Rule]", 
       RowBox[{"Plus", "@@", 
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"Indexed", "[", 
           RowBox[{"\[Iota]", ",", "i"}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"i", ",", 
            RowBox[{"NN", "+", "1"}], ",", 
            RowBox[{"2", "NN"}]}], "}"}]}], "]"}]}]}], ",", 
      RowBox[{"Reverse", "[", 
       RowBox[{
       "eigValRep", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
       "]"}]}], "}"}]}], "//", "Expand"}], "//", "FullSimplify"}]}], "Input",
 CellChangeTimes->{{3.677299029738016*^9, 3.677299067690866*^9}, {
  3.67729910005999*^9, 3.6772991193208227`*^9}, {3.677299202020073*^9, 
  3.6772992023163633`*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"8", " ", 
    SubscriptBox["\[Iota]", "9"], " ", 
    SubscriptBox["\[Lambda]", "1"]}], "-", 
   RowBox[{"we", " ", 
    SubscriptBox["\[CapitalSigma]", "e"]}], "+", 
   RowBox[{"wi", " ", 
    SubscriptBox["\[CapitalSigma]", "i"]}]}], 
  RowBox[{"8", " ", 
   SubscriptBox["\[Lambda]", "1"]}]]], "Output",
 CellChangeTimes->{{3.677299030483445*^9, 3.677299068466289*^9}, {
   3.677299113649909*^9, 3.6772991301313963`*^9}, 3.6772992027159033`*^9, 
   3.67730164710533*^9, 3.6950279415426407`*^9, 3.7184475452349443`*^9}],

Cell[BoxData[
 RowBox[{
  SubscriptBox["\[Iota]", "9"], "\[Equal]", 
  SubscriptBox["\[Iota]", "9"]}]], "Output",
 CellChangeTimes->{{3.677299030483445*^9, 3.677299068466289*^9}, {
   3.677299113649909*^9, 3.6772991301313963`*^9}, 3.6772992027159033`*^9, 
   3.67730164710533*^9, 3.6950279415426407`*^9, 3.718447545268497*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "Derivative of fixed-point response, under several perturbations\n",
 StyleBox["Our approach is to find perturbations of several variables, that \
give rise to a counterintuitive or \[OpenCurlyDoubleQuote]paradoxical\
\[CloseCurlyDoubleQuote] response of the network (ref. Tsodyks 1997). For \
example, driving inhibitory neurons harder has the result of suppressing \
their activity.", "Text"]
}], "Section",
 CellChangeTimes->{{3.6705881853067503`*^9, 3.670588194866437*^9}, {
  3.6706425353504057`*^9, 3.670642606374888*^9}}],

Cell[CellGroupData[{

Cell["Perturbation of entire inhibitory population", "Subsection",
 CellChangeTimes->{{3.670588204490357*^9, 3.6705882114495583`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"FullInhDeriv", "=", 
   RowBox[{
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{"FPSys", "/.", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Indexed", "[", 
           RowBox[{"\[Iota]", ",", "i"}], "]"}], "\[Rule]", 
          SubscriptBox["\[Iota]", "inh"]}], ",", 
         RowBox[{"{", 
          RowBox[{"i", ",", 
           RowBox[{"NN", "+", "1"}], ",", 
           RowBox[{"2", "NN"}]}], "}"}]}], "]"}]}], ",", 
      SubscriptBox["\[Iota]", "inh"]}], "]"}], "//", "FullSimplify"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.670044678789711*^9, 3.670044736951647*^9}, 
   3.670044770422584*^9, 3.670050419395771*^9, {3.670052266035129*^9, 
   3.670052270346363*^9}, {3.670052816434553*^9, 3.670052816594602*^9}, 
   3.670052875844575*^9, {3.670127544915251*^9, 3.6701275489919453`*^9}, {
   3.670583692038272*^9, 3.670583692430435*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Reduce", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"FullInhDeriv", "\[LeftDoubleBracket]", 
      RowBox[{"2", "NN"}], "\[RightDoubleBracket]"}], "<", " ", "0"}], "}"}], 
   "]"}], "//", "FullSimplify"}]], "Input",
 CellChangeTimes->{{3.670044739355278*^9, 3.6700447850863543`*^9}, {
   3.6700454529758863`*^9, 3.670045461077265*^9}, {3.670046575643942*^9, 
   3.670046575739312*^9}, {3.670050431823048*^9, 3.670050432087346*^9}, {
   3.670052338068644*^9, 3.6700523382046127`*^9}, {3.670053365625619*^9, 
   3.670053365841537*^9}, {3.670083300866959*^9, 3.670083301027567*^9}, {
   3.670084495575017*^9, 3.67008449577461*^9}, 3.670127553968556*^9, 
   3.670583697513617*^9, {3.675923994159422*^9, 3.675923994302195*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"1", "+", "wi"}], "<", "we", "<", "1"}], "||", 
  RowBox[{"1", "<", "we", "<", 
   RowBox[{"1", "+", "wi"}]}]}]], "Output",
 CellChangeTimes->{
  3.675923994967407*^9, 3.676007754602316*^9, {3.67730164180164*^9, 
   3.677301647257901*^9}, 3.695027941705522*^9, 3.7184475453491898`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Reduce", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"FullInhDeriv", "\[LeftDoubleBracket]", 
       RowBox[{"2", "NN"}], "\[RightDoubleBracket]"}], "<", " ", "0"}], ",", 
     "StabilityCond"}], "}"}], "]"}], "//", "FullSimplify"}]], "Input",
 CellChangeTimes->{{3.670044739355278*^9, 3.6700447850863543`*^9}, {
   3.6700454529758863`*^9, 3.670045461077265*^9}, {3.670046575643942*^9, 
   3.670046575739312*^9}, {3.670050431823048*^9, 3.670050432087346*^9}, {
   3.670052338068644*^9, 3.6700523382046127`*^9}, {3.670053365625619*^9, 
   3.670053365841537*^9}, {3.670083300866959*^9, 3.670083301027567*^9}, {
   3.670084495575017*^9, 3.67008449577461*^9}, 3.670127553968556*^9, 
   3.670583697513617*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"1", "<", "we", "<", 
   RowBox[{"1", "+", "wi"}]}], "&&", 
  RowBox[{"\[Tau]", ">", "0"}]}]], "Output",
 CellChangeTimes->{{3.6700447497011957`*^9, 3.670044785311227*^9}, {
   3.6700453343429537`*^9, 3.6700453632425413`*^9}, 3.670045461342709*^9, 
   3.670046038946089*^9, 3.670046576243332*^9, 3.670049550953711*^9, {
   3.67005038674736*^9, 3.670050399813513*^9}, 3.670050439298291*^9, 
   3.670050879358572*^9, {3.670052338602292*^9, 3.670052354947538*^9}, 
   3.670052433604939*^9, 3.670052540004456*^9, {3.6700526183919277`*^9, 
   3.67005262963865*^9}, 3.6700527162338743`*^9, {3.670052812381153*^9, 
   3.670052909816915*^9}, 3.670052996814638*^9, 3.670053046172673*^9, 
   3.6700530853473167`*^9, {3.670053240314608*^9, 3.670053243838388*^9}, 
   3.6700533662528543`*^9, 3.6700538493092117`*^9, 3.670054390082477*^9, 
   3.670059053696566*^9, 3.6700593331853943`*^9, {3.670083301423778*^9, 
   3.6700833053393517`*^9}, 3.6700844961270742`*^9, 3.670124202217692*^9, 
   3.6701246634582443`*^9, {3.670124948855241*^9, 3.6701249544065*^9}, 
   3.6701276743816433`*^9, 3.670210523747464*^9, 3.6702355147585793`*^9, 
   3.6705836698305197`*^9, 3.6705837095698347`*^9, 3.670586240137536*^9, 
   3.670588503718227*^9, 3.675922044483026*^9, 3.676007754632557*^9, {
   3.677301641864002*^9, 3.677301647306929*^9}, 3.695027941752503*^9, 
   3.718447545396927*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullInhDeriv", "\[LeftDoubleBracket]", 
  RowBox[{"2", "NN"}], "\[RightDoubleBracket]"}]], "Input",
 CellChangeTimes->{{3.670123843550746*^9, 3.670123851244276*^9}, 
   3.670127567408051*^9, 3.6705837013847103`*^9}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"-", "1"}], "+", "we"}], 
  RowBox[{
   RowBox[{"-", "1"}], "+", "we", "-", "wi"}]]], "Output",
 CellChangeTimes->{
  3.670123851624157*^9, 3.670124202385005*^9, 3.670124663643896*^9, {
   3.670124949034197*^9, 3.670124954530459*^9}, 3.6701276745790358`*^9, 
   3.67021052404747*^9, 3.67023551518992*^9, 3.6705836701264563`*^9, {
   3.670583709748948*^9, 3.670583720005439*^9}, 3.670586240454764*^9, 
   3.670588503845826*^9, 3.6759220445268917`*^9, 3.6760077546774683`*^9, {
   3.677301641905911*^9, 3.677301647343357*^9}, 3.695027941802848*^9, 
   3.718447545447288*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FullInhDeriv", "\[LeftDoubleBracket]", 
   RowBox[{"2", "NN"}], "\[RightDoubleBracket]"}], "/.", 
  "eigValRep"}]], "Input",
 CellChangeTimes->{{3.670123843550746*^9, 3.670123851244276*^9}, 
   3.670127567408051*^9, 3.6705837013847103`*^9, {3.6773016154560623`*^9, 
   3.6773016566696377`*^9}, {3.677301707190106*^9, 3.677301707618784*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"-", "1"}], "+", "we"}], 
  SubscriptBox["\[Lambda]", "1"]]], "Output",
 CellChangeTimes->{
  3.670123851624157*^9, 3.670124202385005*^9, 3.670124663643896*^9, {
   3.670124949034197*^9, 3.670124954530459*^9}, 3.6701276745790358`*^9, 
   3.67021052404747*^9, 3.67023551518992*^9, 3.6705836701264563`*^9, {
   3.670583709748948*^9, 3.670583720005439*^9}, 3.670586240454764*^9, 
   3.670588503845826*^9, 3.6759220445268917`*^9, 3.6760077546774683`*^9, {
   3.677301631718247*^9, 3.6773016568497267`*^9}, 3.6773017080429077`*^9, 
   3.695027941852385*^9, 3.7184475454961023`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Perturbation of single inhibitory neuron", "Subsection",
 CellChangeTimes->{{3.670588215050158*^9, 3.6705882225771627`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"PointDeriv", "=", 
   RowBox[{
    RowBox[{"D", "[", 
     RowBox[{"FPSys", ",", 
      RowBox[{"Indexed", "[", 
       RowBox[{"\[Iota]", ",", 
        RowBox[{"2", "NN"}]}], "]"}]}], "]"}], "//", "FullSimplify"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.6700448506437883`*^9, 3.6700448631641417`*^9}, 
   3.670050428687345*^9, {3.670052278655773*^9, 3.6700523290169067`*^9}, 
   3.670052376529291*^9, 3.670052878148353*^9, 3.6701275519696007`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Reduce", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"PointDeriv", "\[LeftDoubleBracket]", 
       RowBox[{"2", "NN"}], "\[RightDoubleBracket]"}], "<", "0"}], ",", 
     "StabilityCond"}], "}"}], "]"}], "//", "FullSimplify"}]], "Input",
 CellChangeTimes->{{3.6700448916955967`*^9, 3.670044909353031*^9}, {
   3.670045468861899*^9, 3.670045474125636*^9}, {3.670045752620401*^9, 
   3.670045761204282*^9}, {3.670046272979391*^9, 3.670046297081152*^9}, {
   3.6700465723985567`*^9, 3.670046574139526*^9}, {3.670050437656357*^9, 
   3.670050437798821*^9}, {3.6700523342885103`*^9, 3.67005234026051*^9}, {
   3.670052931842369*^9, 3.670052933586488*^9}, {3.670053102269142*^9, 
   3.6700531171706657`*^9}, {3.670053161355844*^9, 3.670053180760365*^9}, {
   3.6700533640659018`*^9, 3.670053364225322*^9}, {3.670054408084519*^9, 
   3.6700544081955843`*^9}, 3.670127555488545*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"1", "+", 
    FractionBox[
     RowBox[{"7", " ", "wi"}], "8"]}], "<", "we", "<", 
   RowBox[{"1", "+", "wi"}]}], "&&", 
  RowBox[{"\[Tau]", ">", "0"}]}]], "Output",
 CellChangeTimes->{
  3.670045761487391*^9, 3.6700460389909887`*^9, {3.670046273433957*^9, 
   3.6700462973312263`*^9}, 3.670046577250676*^9, 3.670049551005803*^9, {
   3.670050386817033*^9, 3.670050399848104*^9}, 3.670050439335989*^9, 
   3.6700508794443207`*^9, {3.670052334603591*^9, 3.670052354997594*^9}, 
   3.670052433652444*^9, 3.670052540048709*^9, {3.6700526184378223`*^9, 
   3.670052629693872*^9}, 3.670052716286766*^9, {3.670052812434619*^9, 
   3.670052933889151*^9}, 3.670052996857421*^9, 3.670053046220269*^9, {
   3.67005308537246*^9, 3.670053117669544*^9}, {3.6700531617798433`*^9, 
   3.670053181144059*^9}, {3.6700532403411407`*^9, 3.6700532438810253`*^9}, 
   3.670053366982443*^9, 3.670053849355283*^9, {3.670054390118752*^9, 
   3.670054415793676*^9}, 3.670059053747253*^9, 3.670059333234798*^9, 
   3.6700833053747387`*^9, 3.670124202273972*^9, 3.670124663510318*^9, {
   3.670124948899981*^9, 3.670124954439403*^9}, 3.670127674435166*^9, 
   3.670210523798306*^9, 3.670235514805382*^9, 3.670583669865603*^9, 
   3.670583709615534*^9, 3.670586240172957*^9, 3.670588504049535*^9, 
   3.675923268086075*^9, 3.676007754792885*^9, {3.6773016421167517`*^9, 
   3.677301647574636*^9}, 3.695027942015942*^9, 3.718447545590726*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"PointDeriv", "\[LeftDoubleBracket]", 
   RowBox[{"2", "NN"}], "\[RightDoubleBracket]"}], "//", 
  "FullSimplify"}]], "Input",
 CellChangeTimes->{{3.670123819742983*^9, 3.67012382800876*^9}, 
   3.670127571055646*^9, {3.677301957433107*^9, 3.677301981111587*^9}, {
   3.677302155197537*^9, 3.6773021582053947`*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{"8", "-", 
   RowBox[{"8", " ", "we"}], "+", 
   RowBox[{"7", " ", "wi"}]}], 
  RowBox[{"8", "-", 
   RowBox[{"8", " ", "we"}], "+", 
   RowBox[{"8", " ", "wi"}]}]]], "Output",
 CellChangeTimes->{
  3.670123828415339*^9, 3.670124202439825*^9, 3.670124663671997*^9, {
   3.670124949116164*^9, 3.670124954563059*^9}, 3.670127674643321*^9, 
   3.670210524081534*^9, 3.670235515275402*^9, 3.670583670176588*^9, 
   3.6705837098013277`*^9, 3.670586240504799*^9, 3.6705885040926867`*^9, 
   3.6759232681501102`*^9, 3.6760077548406143`*^9, {3.6773016421573563`*^9, 
   3.677301647624466*^9}, {3.677301959171138*^9, 3.67730198134855*^9}, 
   3.677302158407299*^9, 3.6950279420693083`*^9, 3.71844754584641*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"1", "+", 
      RowBox[{"wi", "/", 
       RowBox[{"(", 
        RowBox[{"NN", " ", 
         SubscriptBox["\[Lambda]", "1"]}], ")"}]}]}], "/.", 
     RowBox[{"Reverse", "[", 
      RowBox[{
      "eigValRep", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
      "]"}]}], "//", "Factor"}], "//", "Expand"}], "//", 
  "FullSimplify"}]], "Input",
 CellChangeTimes->{{3.677302074622135*^9, 3.677302223470709*^9}, {
  3.6773022545244713`*^9, 3.6773023563305473`*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{"8", "-", 
   RowBox[{"8", " ", "we"}], "+", 
   RowBox[{"7", " ", "wi"}]}], 
  RowBox[{"8", "-", 
   RowBox[{"8", " ", "we"}], "+", 
   RowBox[{"8", " ", "wi"}]}]]], "Output",
 CellChangeTimes->{{3.677302284167313*^9, 3.6773023565279493`*^9}, 
   3.695027942119772*^9, 3.71844754589686*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Perturbation of inhibitory input to all neurons", "Subsection",
 CellChangeTimes->{{3.670588225016788*^9, 3.670588233752727*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"GlobalInhDeriv", "=", 
   RowBox[{
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{"FPSys", "/.", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Indexed", "[", 
           RowBox[{"\[Iota]", ",", "i"}], "]"}], "\[Rule]", 
          SubscriptBox["\[Iota]", "g"]}], ",", 
         RowBox[{"{", 
          RowBox[{"i", ",", "1", ",", 
           RowBox[{"2", "NN"}]}], "}"}]}], "]"}]}], ",", 
      SubscriptBox["\[Iota]", "g"]}], "]"}], "//", "FullSimplify"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.670583726141317*^9, 3.670583773124502*^9}, 
   3.670584350348855*^9, {3.670586115865534*^9, 3.670586174249937*^9}, {
   3.6705862165757113`*^9, 3.670586248462326*^9}, {3.670586349856389*^9, 
   3.670586358856368*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Reduce", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"GlobalInhDeriv", "\[LeftDoubleBracket]", 
       RowBox[{"2", "NN"}], "\[RightDoubleBracket]"}], "<", "0"}], ",", 
     "StabilityCond"}], "}"}], "]"}], "//", "FullSimplify"}]], "Input",
 CellChangeTimes->{{3.6705843531268272`*^9, 3.670584378484111*^9}, {
  3.670584702717415*^9, 3.6705847131165533`*^9}, {3.670585791052864*^9, 
  3.670585797184857*^9}}],

Cell[BoxData["False"], "Output",
 CellChangeTimes->{{3.670584373995989*^9, 3.6705843786815157`*^9}, {
   3.670584703311482*^9, 3.67058471677634*^9}, {3.6705857950688257`*^9, 
   3.670585797631772*^9}, 3.670586240219709*^9, 3.6705885041949167`*^9, 
   3.675923936751957*^9, 3.676007754933527*^9, {3.677301642284988*^9, 
   3.677301647743239*^9}, 3.695027942199196*^9, 3.7184475459836683`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GlobalInhDeriv", "\[LeftDoubleBracket]", 
  RowBox[{"2", "NN"}], "\[RightDoubleBracket]"}]], "Input",
 CellChangeTimes->{{3.6705845691993923`*^9, 3.670584570973179*^9}}],

Cell[BoxData[
 FractionBox["1", 
  RowBox[{"1", "-", "we", "+", "wi"}]]], "Output",
 CellChangeTimes->{
  3.670584571181368*^9, 3.6705862405596247`*^9, 3.670588504243636*^9, 
   3.6759239368223867`*^9, 3.676007754964593*^9, {3.677301642325069*^9, 
   3.6773016477827663`*^9}, 3.6950279422546988`*^9, 3.718447546029894*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Perturbation of inhibitory weight", "Subsection",
 CellChangeTimes->{{3.670588270704171*^9, 3.6705882783995523`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"FPSysEIInput", " ", ":=", " ", 
   RowBox[{"FPSys", "/.", 
    RowBox[{"Join", "[", 
     RowBox[{
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Indexed", "[", 
          RowBox[{"\[Iota]", ",", "i"}], "]"}], "\[Rule]", 
         SubscriptBox["\[Iota]", "e"]}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "1", ",", "NN"}], "}"}]}], "]"}], ",", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Indexed", "[", 
          RowBox[{"\[Iota]", ",", "i"}], "]"}], "\[Rule]", 
         SubscriptBox["\[Iota]", "inh"]}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", 
          RowBox[{"NN", "+", "1"}], ",", 
          RowBox[{"2", "NN"}]}], "}"}]}], "]"}]}], "]"}]}]}], ";", 
  RowBox[{"InhWeightDeriv", "=", 
   RowBox[{
    RowBox[{"D", "[", 
     RowBox[{"FPSysEIInput", ",", "wi"}], "]"}], "//", "FullSimplify"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.670585212279683*^9, 3.6705852315273457`*^9}, 
   3.6705853039493303`*^9, {3.670585590738529*^9, 3.670585670792466*^9}, 
   3.6705861554263287`*^9, 3.670586229296699*^9, {3.670586364264564*^9, 
   3.670586369737733*^9}, {3.670586415552833*^9, 3.6705864304633703`*^9}, {
   3.6705880545435534`*^9, 3.670588054864599*^9}, {3.670588249321175*^9, 
   3.670588249728897*^9}, 3.676011318172987*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Reduce", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"InhWeightDeriv", "\[LeftDoubleBracket]", 
        RowBox[{"2", "NN"}], "\[RightDoubleBracket]"}], ">", "0"}], ",", 
      "StabilityCond", ",", 
      RowBox[{"we", ">", "1"}], ",", 
      RowBox[{
       SubscriptBox["\[Iota]", "e"], ">", "0"}], ",", 
      RowBox[{
       SubscriptBox["\[Iota]", "inh"], ">", "0"}]}], "}"}], ",", 
    SubscriptBox["\[Iota]", "inh"]}], "]"}], "//", "FullSimplify"}]], "Input",\

 CellChangeTimes->{{3.6705868360385637`*^9, 3.670586836421359*^9}, {
  3.670588066924741*^9, 3.670588095563218*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"1", "<", "we", "<", 
   RowBox[{"1", "+", "wi"}]}], "&&", 
  RowBox[{
   SubscriptBox["\[Iota]", "e"], ">", "0"}], "&&", 
  RowBox[{"\[Tau]", ">", "0"}], "&&", 
  RowBox[{
   SubscriptBox["\[Iota]", "inh"], ">", 
   FractionBox[
    RowBox[{"we", " ", 
     SubscriptBox["\[Iota]", "e"]}], 
    RowBox[{
     RowBox[{"-", "1"}], "+", "we"}]]}]}]], "Output",
 CellChangeTimes->{
  3.670586837037916*^9, {3.6705880696502867`*^9, 3.670588098249915*^9}, 
   3.67058850441083*^9, 3.676007755037416*^9, 3.676011320243916*^9, {
   3.6773016425111094`*^9, 3.677301647992408*^9}, 3.69502794239498*^9, 
   3.7184475461759863`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Reduce", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"InhWeightDeriv", "\[LeftDoubleBracket]", 
       RowBox[{"2", "NN"}], "\[RightDoubleBracket]"}], ">", "0"}], ",", 
     "StabilityCond", ",", 
     RowBox[{"we", ">", "1"}], ",", 
     RowBox[{
      SubscriptBox["\[Iota]", "e"], ">", "0"}], ",", 
     RowBox[{
      SubscriptBox["\[Iota]", "inh"], ">", "0"}], ",", 
     RowBox[{
      RowBox[{"FPSysEIInput", "\[LeftDoubleBracket]", 
       RowBox[{"2", "NN"}], "\[RightDoubleBracket]"}], ">", "0"}]}], "}"}], 
   "]"}], "//", "FullSimplify"}]], "Input",
 CellChangeTimes->{{3.670585308187118*^9, 3.670585529185235*^9}, {
   3.670585560781142*^9, 3.670585571042595*^9}, {3.670585684896358*^9, 
   3.6705857093187647`*^9}, {3.67058579943388*^9, 3.670585818426016*^9}, {
   3.6705859614509563`*^9, 3.670585962382661*^9}, 3.670585999390547*^9, {
   3.6705860612674294`*^9, 3.670586071722746*^9}, {3.670586439266086*^9, 
   3.67058645172569*^9}, {3.6705871739668827`*^9, 3.67058717439791*^9}, {
   3.6705880738252563`*^9, 3.6705881058669357`*^9}, 3.6705882570916843`*^9}],

Cell[BoxData["False"], "Output",
 CellChangeTimes->{{3.670585692614648*^9, 3.670585709929595*^9}, {
   3.670585800021353*^9, 3.67058581872369*^9}, 3.670585965391341*^9, 
   3.670586000162801*^9, 3.670586073005517*^9, 3.6705862403290977`*^9, {
   3.6705864350682993`*^9, 3.670586452193186*^9}, 3.670587177087888*^9, {
   3.670588075531949*^9, 3.67058810633248*^9}, 3.6705882641677313`*^9, 
   3.670588504492622*^9, 3.676007755242489*^9, 3.676011321460713*^9, {
   3.67730164266049*^9, 3.677301648111195*^9}, 3.695027942452877*^9, 
   3.7184475462367373`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
  "FPSysEIInput", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], "//",
   "FullSimplify"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"FPSysEIInput", "\[LeftDoubleBracket]", 
   RowBox[{"2", "NN"}], "\[RightDoubleBracket]"}], "//", 
  "FullSimplify"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
   "FPSysEIInput", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], "/.",
    "eigValRep"}], "//", "FullSimplify"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"FPSysEIInput", "\[LeftDoubleBracket]", 
    RowBox[{"2", "NN"}], "\[RightDoubleBracket]"}], "/.", "eigValRep"}], "//",
   "FullSimplify"}]}], "Input",
 CellChangeTimes->{{3.676014974944275*^9, 3.6760149888072987`*^9}, {
  3.677301929015065*^9, 3.677301936110506*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "+", "wi"}], ")"}], " ", 
    SubscriptBox["\[Iota]", "e"]}], "-", 
   RowBox[{"wi", " ", 
    SubscriptBox["\[Iota]", "inh"]}]}], 
  RowBox[{"1", "-", "we", "+", "wi"}]]], "Output",
 CellChangeTimes->{
  3.676014642901071*^9, {3.676014978045991*^9, 3.67601498938832*^9}, {
   3.6773016429592457`*^9, 3.6773016482217503`*^9}, 3.6773019365008917`*^9, 
   3.695027942523426*^9, 3.71844754631209*^9}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"we", " ", 
    SubscriptBox["\[Iota]", "e"]}], "+", 
   SubscriptBox["\[Iota]", "inh"], "-", 
   RowBox[{"we", " ", 
    SubscriptBox["\[Iota]", "inh"]}]}], 
  RowBox[{"1", "-", "we", "+", "wi"}]]], "Output",
 CellChangeTimes->{
  3.676014642901071*^9, {3.676014978045991*^9, 3.67601498938832*^9}, {
   3.6773016429592457`*^9, 3.6773016482217503`*^9}, 3.6773019365008917`*^9, 
   3.695027942523426*^9, 3.718447546372259*^9}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     RowBox[{"(", 
      RowBox[{"1", "+", "wi"}], ")"}]}], " ", 
    SubscriptBox["\[Iota]", "e"]}], "+", 
   RowBox[{"wi", " ", 
    SubscriptBox["\[Iota]", "inh"]}]}], 
  SubscriptBox["\[Lambda]", "1"]]], "Output",
 CellChangeTimes->{
  3.676014642901071*^9, {3.676014978045991*^9, 3.67601498938832*^9}, {
   3.6773016429592457`*^9, 3.6773016482217503`*^9}, 3.6773019365008917`*^9, 
   3.695027942523426*^9, 3.7184475464305353`*^9}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", "we"}], " ", 
    SubscriptBox["\[Iota]", "e"]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", "we"}], ")"}], " ", 
    SubscriptBox["\[Iota]", "inh"]}]}], 
  SubscriptBox["\[Lambda]", "1"]]], "Output",
 CellChangeTimes->{
  3.676014642901071*^9, {3.676014978045991*^9, 3.67601498938832*^9}, {
   3.6773016429592457`*^9, 3.6773016482217503`*^9}, 3.6773019365008917`*^9, 
   3.695027942523426*^9, 3.7184475464879017`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"InhWeightDeriv", "\[LeftDoubleBracket]", "1", 
  "\[RightDoubleBracket]"}], "\[IndentingNewLine]", 
 RowBox[{"InhWeightDeriv", "\[LeftDoubleBracket]", 
  RowBox[{"2", "NN"}], "\[RightDoubleBracket]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
  "InhWeightDeriv", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], "/.",
   "eigValRep"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"InhWeightDeriv", "\[LeftDoubleBracket]", 
   RowBox[{"2", "NN"}], "\[RightDoubleBracket]"}], "/.", 
  "eigValRep"}]}], "Input",
 CellChangeTimes->{{3.670586954634963*^9, 3.670586957662087*^9}, {
  3.676015102672015*^9, 3.676015105227743*^9}, {3.677301856098381*^9, 
  3.677301861229313*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", "we"}], " ", 
    SubscriptBox["\[Iota]", "e"]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", "we"}], ")"}], " ", 
    SubscriptBox["\[Iota]", "inh"]}]}], 
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{"1", "-", "we", "+", "wi"}], ")"}], "2"]]], "Output",
 CellChangeTimes->{
  3.676015117630011*^9, {3.6773016430594893`*^9, 3.67730164834722*^9}, 
   3.677301861532076*^9, 3.677301910142397*^9, 3.695027942740177*^9, 
   3.718447546532179*^9}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", "we"}], " ", 
    SubscriptBox["\[Iota]", "e"]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", "we"}], ")"}], " ", 
    SubscriptBox["\[Iota]", "inh"]}]}], 
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{"1", "-", "we", "+", "wi"}], ")"}], "2"]]], "Output",
 CellChangeTimes->{
  3.676015117630011*^9, {3.6773016430594893`*^9, 3.67730164834722*^9}, 
   3.677301861532076*^9, 3.677301910142397*^9, 3.695027942740177*^9, 
   3.7184475465334806`*^9}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", "we"}], " ", 
    SubscriptBox["\[Iota]", "e"]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", "we"}], ")"}], " ", 
    SubscriptBox["\[Iota]", "inh"]}]}], 
  SubsuperscriptBox["\[Lambda]", "1", "2"]]], "Output",
 CellChangeTimes->{
  3.676015117630011*^9, {3.6773016430594893`*^9, 3.67730164834722*^9}, 
   3.677301861532076*^9, 3.677301910142397*^9, 3.695027942740177*^9, 
   3.718447546534911*^9}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", "we"}], " ", 
    SubscriptBox["\[Iota]", "e"]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", "we"}], ")"}], " ", 
    SubscriptBox["\[Iota]", "inh"]}]}], 
  SubsuperscriptBox["\[Lambda]", "1", "2"]]], "Output",
 CellChangeTimes->{
  3.676015117630011*^9, {3.6773016430594893`*^9, 3.67730164834722*^9}, 
   3.677301861532076*^9, 3.677301910142397*^9, 3.695027942740177*^9, 
   3.718447546535982*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
   "FPSysEIInput", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[Iota]", "e"], "\[Rule]", 
      SubscriptBox["\[Iota]", "g"]}], ",", 
     RowBox[{
      SubscriptBox["\[Iota]", "inh"], "\[Rule]", 
      SubscriptBox["\[Iota]", "g"]}]}], "}"}]}], "//", 
  "FullSimplify"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"FPSysEIInput", "\[LeftDoubleBracket]", 
    RowBox[{"2", "NN"}], "\[RightDoubleBracket]"}], "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[Iota]", "e"], "\[Rule]", 
      SubscriptBox["\[Iota]", "g"]}], ",", 
     RowBox[{
      SubscriptBox["\[Iota]", "inh"], "\[Rule]", 
      SubscriptBox["\[Iota]", "g"]}]}], "}"}]}], "//", 
  "FullSimplify"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
    "FPSysEIInput", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], "/.",
     "eigValRep"}], "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[Iota]", "e"], "\[Rule]", 
      SubscriptBox["\[Iota]", "g"]}], ",", 
     RowBox[{
      SubscriptBox["\[Iota]", "inh"], "\[Rule]", 
      SubscriptBox["\[Iota]", "g"]}]}], "}"}]}], "//", 
  "FullSimplify"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"FPSysEIInput", "\[LeftDoubleBracket]", 
     RowBox[{"2", "NN"}], "\[RightDoubleBracket]"}], "/.", "eigValRep"}], "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[Iota]", "e"], "\[Rule]", 
      SubscriptBox["\[Iota]", "g"]}], ",", 
     RowBox[{
      SubscriptBox["\[Iota]", "inh"], "\[Rule]", 
      SubscriptBox["\[Iota]", "g"]}]}], "}"}]}], "//", 
  "FullSimplify"}]}], "Input",
 CellChangeTimes->{{3.670587123328265*^9, 3.670587125981778*^9}, {
  3.6705875066647577`*^9, 3.670587516279011*^9}, {3.6705883849738913`*^9, 
  3.670588415474173*^9}, {3.676015529293214*^9, 3.676015543555078*^9}, {
  3.6773036541616793`*^9, 3.677303658786471*^9}}],

Cell[BoxData[
 FractionBox[
  SubscriptBox["\[Iota]", "g"], 
  RowBox[{"1", "-", "we", "+", "wi"}]]], "Output",
 CellChangeTimes->{
  3.6705871264413347`*^9, 3.670587516603828*^9, 3.670588415853697*^9, 
   3.6705885046783857`*^9, 3.676007755575409*^9, 3.67601132366324*^9, {
   3.6760155345764303`*^9, 3.676015544045148*^9}, {3.677301643160138*^9, 
   3.67730164843546*^9}, 3.6773036591656313`*^9, 3.695027942840405*^9, 
   3.7184475465962763`*^9}],

Cell[BoxData[
 FractionBox[
  SubscriptBox["\[Iota]", "g"], 
  RowBox[{"1", "-", "we", "+", "wi"}]]], "Output",
 CellChangeTimes->{
  3.6705871264413347`*^9, 3.670587516603828*^9, 3.670588415853697*^9, 
   3.6705885046783857`*^9, 3.676007755575409*^9, 3.67601132366324*^9, {
   3.6760155345764303`*^9, 3.676015544045148*^9}, {3.677301643160138*^9, 
   3.67730164843546*^9}, 3.6773036591656313`*^9, 3.695027942840405*^9, 
   3.7184475466729183`*^9}],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox[
   SubscriptBox["\[Iota]", "g"], 
   SubscriptBox["\[Lambda]", "1"]]}]], "Output",
 CellChangeTimes->{
  3.6705871264413347`*^9, 3.670587516603828*^9, 3.670588415853697*^9, 
   3.6705885046783857`*^9, 3.676007755575409*^9, 3.67601132366324*^9, {
   3.6760155345764303`*^9, 3.676015544045148*^9}, {3.677301643160138*^9, 
   3.67730164843546*^9}, 3.6773036591656313`*^9, 3.695027942840405*^9, 
   3.7184475467739687`*^9}],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox[
   SubscriptBox["\[Iota]", "g"], 
   SubscriptBox["\[Lambda]", "1"]]}]], "Output",
 CellChangeTimes->{
  3.6705871264413347`*^9, 3.670587516603828*^9, 3.670588415853697*^9, 
   3.6705885046783857`*^9, 3.676007755575409*^9, 3.67601132366324*^9, {
   3.6760155345764303`*^9, 3.676015544045148*^9}, {3.677301643160138*^9, 
   3.67730164843546*^9}, 3.6773036591656313`*^9, 3.695027942840405*^9, 
   3.718447546775156*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
   "FPSysEIInput", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], "/.", 
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["\[Iota]", "inh"], "\[Rule]", 
     RowBox[{"we", " ", 
      RowBox[{
       SubscriptBox["\[Iota]", "e"], "/", 
       RowBox[{"(", 
        RowBox[{"we", "-", "1"}], ")"}]}]}]}], "}"}]}], "//", 
  "FullSimplify"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"FPSysEIInput", "\[LeftDoubleBracket]", 
    RowBox[{"2", "NN"}], "\[RightDoubleBracket]"}], "/.", 
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["\[Iota]", "inh"], "\[Rule]", 
     RowBox[{"we", " ", 
      RowBox[{
       SubscriptBox["\[Iota]", "e"], "/", 
       RowBox[{"(", 
        RowBox[{"we", "-", "1"}], ")"}]}]}]}], "}"}]}], "//", 
  "FullSimplify"}]}], "Input",
 CellChangeTimes->{{3.676014551572548*^9, 3.6760145985575037`*^9}, {
   3.6760154976413517`*^9, 3.676015549956567*^9}, {3.676015583492483*^9, 
   3.676015620093758*^9}, {3.676015660542061*^9, 3.676015662362816*^9}, 
   3.677301807539112*^9}],

Cell[BoxData[
 FractionBox[
  SubscriptBox["\[Iota]", "e"], 
  RowBox[{"1", "-", "we"}]]], "Output",
 CellChangeTimes->{
  3.6760145227120028`*^9, {3.676014594930211*^9, 3.676014598808351*^9}, {
   3.676015498522127*^9, 3.676015550715705*^9}, 3.676015593791404*^9, 
   3.676015630353964*^9, 3.676015662778576*^9, {3.6773016433205557`*^9, 
   3.677301648579727*^9}, 3.6773018092086887`*^9, 3.695027943354773*^9, 
   3.718447546981367*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{
  3.6760145227120028`*^9, {3.676014594930211*^9, 3.676014598808351*^9}, {
   3.676015498522127*^9, 3.676015550715705*^9}, 3.676015593791404*^9, 
   3.676015630353964*^9, 3.676015662778576*^9, {3.6773016433205557`*^9, 
   3.677301648579727*^9}, 3.6773018092086887`*^9, 3.695027943354773*^9, 
   3.718447547043663*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "Analysis of general conditions permitting paradoxical response while \
driving defined portion of inhibitory network\n",
 StyleBox["Result obtained through observation of dynamics under various ", 
  "Text"],
 StyleBox["NN", "Input"],
 StyleBox[" and various ", "Text"],
 StyleBox["p", "Input"],
 StyleBox[".", "Text"]
}], "Section",
 CellChangeTimes->{{3.670588308278867*^9, 3.6705883375994787`*^9}, {
  3.670588443090358*^9, 3.670588462624612*^9}, {3.670642617862431*^9, 
  3.670642627836873*^9}, {3.670642706362652*^9, 3.670642718937818*^9}}],

Cell[CellGroupData[{

Cell["\<\
Conditions for paradoxical response when driving a single inhibitory neuron\
\>", "Subsection",
 CellChangeTimes->{{3.6706427232366962`*^9, 3.670642735721532*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"GeneralPointParadoxConds", "[", "NN_", "]"}], ":=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"1", "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"NN", "-", "1"}], ")"}], 
      RowBox[{"wi", "/", "NN"}]}]}], ",", 
    RowBox[{"1", "+", "wi"}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.670053245504381*^9, 3.670053297174569*^9}, {
   3.670053336782566*^9, 3.670053337069335*^9}, {3.6700533697546062`*^9, 
   3.6700533699207*^9}, {3.6700535050933847`*^9, 3.670053513298606*^9}, 
   3.670053733849353*^9, {3.6701275612722387`*^9, 3.6701275650476522`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"GeneralPointParadoxConds", "[", "x", "]"}], "/.", 
    RowBox[{"wi", "\[Rule]", "1"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "1", ",", "100"}], "}"}], ",", 
   RowBox[{"AxesOrigin", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"1", ",", "0"}], "}"}]}], ",", 
   RowBox[{"PlotRange", "\[Rule]", "Full"}], ",", 
   RowBox[{"AxesLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
     "\"\<Network size\>\"", ",", 
      "\"\<Constraints on we in multiples of wi\>\""}], "}"}]}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.670053396002042*^9, 3.67005348812158*^9}, {
  3.670053519403054*^9, 3.670053530730228*^9}, {3.670053568656847*^9, 
  3.670053756174355*^9}, {3.670053855560652*^9, 3.670053885060789*^9}, {
  3.670053916677846*^9, 3.670053938098866*^9}, {3.670053969969537*^9, 
  3.67005397227355*^9}, {3.670054422996847*^9, 3.670054437011355*^9}}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwl1nk4ldv3APCTIUPJLNXBIY7jXJ3IELfLWgmlkrEc0iQhupJkKLdEVKZK
bkVlilKpkBA3REiI6hiaSJNCMg/HOe/7fX/P74/9rOfzPHs9a++91h9b0+ug
8z4RGo3WQa3/i5u83dVFaCNwjP7/8XhOrMruyBE46jvv6aPwESj+VLSoamYE
xk4P+EQNjIDqDmkyYnQUXlz2jUirHIXNV02nu/4eA/5tEe++ZWNw4q3Xb5OB
MYhxUl21OmwMvm+t7B39PA60G4eyTzDGYWlqf5fDrgl4pHOmVPngONi/Vmy/
934CrswcvSlSMQ4lWw7U7OdNwq6XMdvf20zAj8Qr5Y2OU+BdOH7D9PQELGt+
VqjTOgV56RrTz+snIHq9WnZf/TSw8uwvrl0zCaWxdmlgNQO3DrrEhQZPws+6
IxeuV83A1nHlM1G3JsERX550L5uFsCXLYy8smIJTx/kRZUZ8YL/ZnPLqzyko
+48ZrFzIp/JvWLr7TYG6+QmvV/lz8CFe0fZa1RQ4h931WKkjAKbtYcno/imI
fdTlnJQtgA7e8uNistMwZGiwzi5dCOL3yo/peExDBbtv+dMEEioCvxrGfp6G
0JL5ybJiNAyvNF8nITID59/Q4y2taCiz0c3pCmMGnsnbRV+roqHJRU+u/PYZ
YCdnB3PL5qF29KJ474YZsL5XFnh6ch5uSF362q93Bna2tPqXGong9fdr9Gun
Z+CCNN9LqVAEF+Dv6mTmLMzEObu054viNX8iZfU/s6Bw08+B/C6KtuazLucu
zIJ+/fFNHB0xHDStP74/bxZ2id5dl5gthj8qdNpLm2eh4YSo8YZ0cSwZYPsz
FfnwKWOJQVi3OF59PKMlps0H/pOV+jdV5mNSBXyLNubDCsF2bbGL8/GohHXT
QVc+pIaXKNUkSGBxn/Dl1vN8kN6Rpcxuk8B7h3WNKzL5cHJtokqqgiTu1UrY
+/w+HwKlvVX90iQx6skS41/NfNhwTZEud0sKS/cZP8sUmYOqKJJ+dEAKs8w7
k9Jl58Bk36Da1xXSuOjrsePO9DnQ4tRplJdI41h2zZN5JnMgqA5evrt2AUpK
zPSc3zsHwbk7tV+IL8TdJkO5+YFz8PPMRh1ju4UYfbUxPiFiDjqdtHQl2xei
68XG6PLkOSj88opd+FEGHwwO9vBK58BbwnAVbVYWGUzLDg1yDn4dSV5ioi6H
tzRZGickBMDyE1s6AHIoYeV6r1JWAJmbR5a6xsjhR1eLn20aAkhUfk5nScvj
yNjD4qOWAvC5FcZoX6yANlGDi4+ECiA77Rcj1lwBo2mKg2SkAD4k7NX8c7sC
9jT5lAXHCMAlyEErN0MBRVJO2SicFwCa62qH6yhiItewxDZfAEubO3UZRkpY
VnHE6DuPco7QJc1ZCTnGmdtevhXAsgjtKIVgJUztDt15q0cAdN3gbrEiJWyM
GV654ocANGJkzv7kKCNLPkX6HF8A2musB4rZKnizCz0+0YWgrRCgor9RBe2q
82gTDCHo/LxglbdfBYkLg0YCbSEwL/emX85XwZdtY+SAvhD0xo9uimQuRnbT
shXr/xIC525RgY2WKoazrRrV3SlHd3dXrVVFbcknWec8hbDSnRQz26OKB14P
X57eJQQDCfsd7CxVjDpX5nvPRwhGe3/IyKovwdST6zVLDgvBbJnGwe4lS1Fi
7cO0DYlCWJeQaBggT0fJjnfnbMuFUHB12tHWgI6dLmmx9RVCUCnwCtJ0oGOC
a4g0PBHCjxazB12JdAwPkpPUqhVC4qJv+taSaii+6e/C281C4F2wYKmRauhb
21XZ+lEI3leG1duGNLC528O8hRCCD09fo0Ocgd7Kk7sqaARc9fx830qZgSyT
mNe5IgSI/W3/Us2YgVmazfWB8wnoTFouwzvEQKu6eLFuGQKOtrXF4y8GqnEE
eyXpBNQ6s+KWfNXES/0aPDVTArZsVuzqmNDE9p+a/z5fTcA7G0I3RVwLo3Yk
WASZEzBqxmuS1tXCao8On9K/CGBoRMkI9mshL2eNmMY6Av4Z7Ert+a2F4Qkn
nPwcCDA7dfrGjbnlOGLWISrjQ8Czf4Indi3Uxuf37pvv9SXAMWyHDV1NG3M3
yJqW+RHg62/Un2qpjfHssCvcAAIuOfay405qY0D+/vSTQQRM0FcX+0no4Pf5
Y4sOHSXgwaPvNSsUmajBdQn4nkhAhuYd3RYtJsZ3DlgNJRGQmPR3sv8qJuae
EL8zkkyA376J7becmJhR/m7V5HkCtJRFphnnmTgi0nryVypVL0RthZKMLk4J
Gs9fv0ZApJFrGn8+C6USLNtk7hLgn7mYlqbCwi1Tjz8NUHZf8N5nNZOFqlXI
ayggwPTLHuMQGxZmnZPzP3afgN8pB9t+xbBwqvh7QlcRAV5j8eJ9ND0caD0V
tKOcALuip4ca+Xo4PGYqLHhGQHJyRVrJAjbqqbz7EVFPAC/g4dNsOhuzal+9
sGkgYDczTz7Sko3ZvQ0R7xoJCL16psggmo3n5D9XTL8gICfWYTRN6g901IkK
Fn1FgMD9Y9ABVX2sTc1ZeOADAVarO6+46+nj0ISW/bKPBJxWaqux/VMf27eG
VjZRVmirkdP01MctvmG62r0E6FnnFnZk6uMdA7t9bX0EbOMEjFjqrsCvb3cV
0voJKBSZPShvysETo7ScrhEC7gguXnhgy8EVgoLQA6ME5E5xHtq7cXDnU6d8
2hgBlwe8p86Gc5DhU8jUHSfg+Ov2SJFKDno7BhUcmCRgc07+2XFYiTMmthXv
Zql+MwpeDWgboNywt4+dCAkXarYPtnAM8c4jxpZMeRJqQpUlkkwN8VGYww1p
BRKG9du07C0N8UflvCtHKG9Ms/JotTfESa/pPRsVSRAJ1mtqPWCIH3q14n4r
kRCyfPrmyzuG+FU1PJKlSgI3LsWrnbkKS5t8dnDVSWBsbnr7Rt0Ii2JZOxr1
SOBXqG1bo2uEHim5njpsEnh6wa9zVhphv26jVAzlMxLLmg+hERY7PQi0+IOE
37UH/pP1MkLvQ/cD7+uTUGUul7kpzwjFFuvlnFxJwnZdN++6P4wx5OzNs7+N
Sbgk8m242MwE1Zl3hwWWJGTf0QlNQBMs/2u5gTWQUODsI9y7wQTvSShYJFCu
y+lfoMw1QXb4mgpVJGHEaoAVFmaCU/7zNhiuJcEu5rfXmlITFDHw/cxdR4JA
lN9ZZ2SKBo8b4mPWk+AlLlvN46zG8qzYSCcHEkoyIuuUl5ljlurusreeJAx5
XRnmGK7B8vUeljv3k3DscC0vfrkFJoQXRwYepurdT++rAsCdp2yihiJJCMXt
8lK3EXcJrQsVYknIdAiSdc1ai+e//HRpo/4la0I+MBIKrFArx+cP1xSqH9Vt
577cXIep7vI2TZdJsNBYqsAqtcZnHhuN1K+TULkxjhNZY4O8D/H93dkkeF+L
yB8tt0VOz7qAnjwSokSfmbg0rMf5Puz897dJaDz/L6Pz6QY82sJ7bnuPBKef
Z/L8yu3wikHxm12FJGxxe2PRX7cR3cQZ1489pM47mGt0+skmzKm9pKdfSoLC
v61/KbZsxq1nybq+chJGpVQ/3e+wx5C1Df0elSTM5f78FNC6BZnRU7SYJySE
b1WYVHrngJFXc9oeVpMgp7Ln4UF/R+T5eE5lPCVBvN/usec3RwyuiW/xrqPe
16rnebW/E351qx8zrqfyDU1PG/U7oULEj9zWBhKm7pg7qAU6o8pU5rag5ySc
LaM1LB5yxo3r3eBzEzU/t7ul0n1dML9kftGfzdQ8MD3dUn65YPS16cPXW0go
NL3hctXfFet9EnW6W0kIfPWv+9IxV/QMSEqjt5HQ6pSmrxm2Fb/PDKRsaifB
at/u+2bjW9Hqnam1+ysSgkTfmDdEbMO7famJQa+p8xpLhXhPb8OdLy5X739D
Qtomx2PxwW646mH8kAuPBLNfVonMGTf0ivT1XN9BzYvm7KY3IVzkkNU6mp0k
vHS7RZChXHR8ENOjTbkwaWuRfgQXWYsyLrEoh8wWqcT9w8WdiV1iBpQF7f59
ZnHU/lCvt5aUZaI+hGZc5qKVLunlSZnTU529v4KLZ/hnZy9RllMKdL30Hxdv
Po64nk55zI4uUVfFxacF9ZhBufRR+AF6HRdT3gfH5lG2TDJc3dbMxUtyuhIl
lLf8daPF+CMXS7bQh9opGwQ7ndjTy8Upvy+xPMoK+aRhch8XP3hJaXRT7lT0
vNz/jYshGcEOvZR3Dip5pQ9zsbitKv8XZdSsU2ocoe4TfdJilLKW26HG8TEu
jnjnvpqg/L22Vd9+movNuxfMzFF+PnOsN2KWi6GtDvEk5TscdsrNOS42yE0u
E+0iIdG72/qNkIubXecK5lM+mB43TZJcjHrqZSFN+X++/2v5
     "]], 
    LineBox[CompressedData["
1:eJxTTMoPSmViYGAwAWIQ7Z0SKcfE8MGeAQIcdmk9VD7Y/R/On1KxReRAN7sD
jC91+pq6grEInO/c3WOYLSgD5x8K0miTfKII56/f+uyArrAanO+58WDh8V+a
cP4Gpp/5gmZ6cP7EA9Gvz+gZwvkKPidvXpYzhvOnMT19t8nCFM5PYuXff0XP
HM7fMq/msKi0JZz/JmnGOz1Dazi/uvjQlS5lWzj/w7pZD/fZ28P5ZQ7Rgpwr
HeD8+f4F/CELHOF865I7Ct1rnOB8pv3n+x8vc4bzbeWlhDS2ucD5u73a9GoO
uML5KXMqV3zc4QbnNzAfMQ0+5g7nH58wVeHaQQ84P/Blx9KMHZ5wvl/4Zdvn
h70Q7n29xLh9rzecLzT1rI3wGR84/yOnxIN1V33h/N9LXj7IPusH51eECn0V
ueUP5wuIJW7OzwqA81mfe+6MeYrgb3G6d2J/ViBCv6FZu/FzBP/bKkt/2bwg
OL9zO8Mx8TcI/r6VNzhnpQfD+WvUYsInvUXwN5gtDp6dFQLn512cGin1CcE/
GzhTR7E8FM53Sk1YZ/EZwS9gvmx5rDIM4V4TzpKU7wj+TO+A6q6icDjf4q1T
j9oPBP+N4k/vyyURcL7evf0LM3ch+Pmz2r7//4/gAwC6ucdP
     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{
    FormBox["\"Network size\"", TraditionalForm], 
    FormBox["\"Constraints on we in multiples of wi\"", TraditionalForm]},
  AxesOrigin->{1, 0},
  Method->{},
  PlotRange->{{1, 100}, {0, 2.}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{{3.6700536413580847`*^9, 3.670053756370858*^9}, 
   3.670053849418788*^9, 3.670053887125746*^9, {3.670053920049395*^9, 
   3.67005393843373*^9}, 3.670053986604444*^9, 3.670054390198618*^9, 
   3.670054425935945*^9, 3.6700590538288717`*^9, 3.670059333319579*^9, 
   3.670083305464081*^9, 3.6701242023462353`*^9, 3.670124663582803*^9, {
   3.670124948981492*^9, 3.6701249544882717`*^9}, 3.6701276745300198`*^9, 
   3.670210524014271*^9, 3.670235515092388*^9, 3.67058367008123*^9, 
   3.6705837096961393`*^9, 3.6705862404155073`*^9, 3.6705885049373207`*^9, 
   3.67600775580716*^9, 3.677301648806493*^9, 3.69502794357981*^9, 
   3.7184475472254343`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "Conditions for paradoxical response when driving a proportion ",
 StyleBox["p", "Input"],
 " of the network\n",
 StyleBox["Solve the partial derivatives in response to a perturbation, for a \
given network", "Text"]
}], "Subsection",
 CellChangeTimes->{{3.6706427388897953`*^9, 3.670642753449432*^9}, {
  3.718447875034091*^9, 3.7184479306529408`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PartialDeriv", "=", 
  RowBox[{
   RowBox[{"D", "[", 
    RowBox[{
     RowBox[{"FPSys", "/.", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Indexed", "[", 
          RowBox[{"\[Iota]", ",", "i"}], "]"}], "\[Rule]", 
         SubscriptBox["\[Iota]", "inh"]}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", 
          RowBox[{
           RowBox[{"2", "NN"}], "-", "2"}], ",", 
          RowBox[{"2", "NN"}]}], "}"}]}], "]"}]}], ",", 
     SubscriptBox["\[Iota]", "inh"]}], "]"}], "//", 
   "FullSimplify"}]}]], "Input",
 CellChangeTimes->{{3.670044678789711*^9, 3.670044736951647*^9}, 
   3.670044770422584*^9, 3.670050419395771*^9, {3.670052266035129*^9, 
   3.670052270346363*^9}, {3.670052816434553*^9, 3.670052816594602*^9}, 
   3.670052875844575*^9, {3.67012388267776*^9, 3.6701238873225718`*^9}, {
   3.670123921242754*^9, 3.670123932696267*^9}, 3.670123973261827*^9, {
   3.670124037468767*^9, 3.670124072995727*^9}, {3.6701241682892447`*^9, 
   3.670124181496871*^9}, {3.670124225751277*^9, 3.6701242273439703`*^9}, {
   3.6701246003059177`*^9, 3.6701246130089827`*^9}, {3.670124743569412*^9, 
   3.670124769269206*^9}, {3.6701248185398827`*^9, 3.670124876007284*^9}, {
   3.670124912669704*^9, 3.67012496326099*^9}, {3.670127574607861*^9, 
   3.6701275758637743`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"3", " ", "wi"}], 
     RowBox[{"8", "-", 
      RowBox[{"8", " ", "we"}], "+", 
      RowBox[{"8", " ", "wi"}]}]]}], ",", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{"3", " ", "wi"}], 
     RowBox[{"8", "-", 
      RowBox[{"8", " ", "we"}], "+", 
      RowBox[{"8", " ", "wi"}]}]]}], ",", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{"3", " ", "wi"}], 
     RowBox[{"8", "-", 
      RowBox[{"8", " ", "we"}], "+", 
      RowBox[{"8", " ", "wi"}]}]]}], ",", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{"3", " ", "wi"}], 
     RowBox[{"8", "-", 
      RowBox[{"8", " ", "we"}], "+", 
      RowBox[{"8", " ", "wi"}]}]]}], ",", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{"3", " ", "wi"}], 
     RowBox[{"8", "-", 
      RowBox[{"8", " ", "we"}], "+", 
      RowBox[{"8", " ", "wi"}]}]]}], ",", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{"3", " ", "wi"}], 
     RowBox[{"8", "-", 
      RowBox[{"8", " ", "we"}], "+", 
      RowBox[{"8", " ", "wi"}]}]]}], ",", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{"3", " ", "wi"}], 
     RowBox[{"8", "-", 
      RowBox[{"8", " ", "we"}], "+", 
      RowBox[{"8", " ", "wi"}]}]]}], ",", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{"3", " ", "wi"}], 
     RowBox[{"8", "-", 
      RowBox[{"8", " ", "we"}], "+", 
      RowBox[{"8", " ", "wi"}]}]]}], ",", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{"3", " ", "wi"}], 
     RowBox[{"8", "-", 
      RowBox[{"8", " ", "we"}], "+", 
      RowBox[{"8", " ", "wi"}]}]]}], ",", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{"3", " ", "wi"}], 
     RowBox[{"8", "-", 
      RowBox[{"8", " ", "we"}], "+", 
      RowBox[{"8", " ", "wi"}]}]]}], ",", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{"3", " ", "wi"}], 
     RowBox[{"8", "-", 
      RowBox[{"8", " ", "we"}], "+", 
      RowBox[{"8", " ", "wi"}]}]]}], ",", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{"3", " ", "wi"}], 
     RowBox[{"8", "-", 
      RowBox[{"8", " ", "we"}], "+", 
      RowBox[{"8", " ", "wi"}]}]]}], ",", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{"3", " ", "wi"}], 
     RowBox[{"8", "-", 
      RowBox[{"8", " ", "we"}], "+", 
      RowBox[{"8", " ", "wi"}]}]]}], ",", 
   FractionBox[
    RowBox[{"8", "-", 
     RowBox[{"8", " ", "we"}], "+", 
     RowBox[{"5", " ", "wi"}]}], 
    RowBox[{"8", "-", 
     RowBox[{"8", " ", "we"}], "+", 
     RowBox[{"8", " ", "wi"}]}]], ",", 
   FractionBox[
    RowBox[{"8", "-", 
     RowBox[{"8", " ", "we"}], "+", 
     RowBox[{"5", " ", "wi"}]}], 
    RowBox[{"8", "-", 
     RowBox[{"8", " ", "we"}], "+", 
     RowBox[{"8", " ", "wi"}]}]], ",", 
   FractionBox[
    RowBox[{"8", "-", 
     RowBox[{"8", " ", "we"}], "+", 
     RowBox[{"5", " ", "wi"}]}], 
    RowBox[{"8", "-", 
     RowBox[{"8", " ", "we"}], "+", 
     RowBox[{"8", " ", "wi"}]}]]}], "}"}]], "Output",
 CellChangeTimes->{{3.670123921856658*^9, 3.670123934549653*^9}, 
   3.670123973869987*^9, {3.670124037692727*^9, 3.6701240732709637`*^9}, {
   3.670124168562435*^9, 3.670124227657943*^9}, {3.670124600984551*^9, 
   3.670124613769011*^9}, 3.670124663733423*^9, {3.67012474381398*^9, 
   3.670124769601512*^9}, {3.6701248190167837`*^9, 3.670124876367189*^9}, {
   3.670124912936092*^9, 3.670124963737969*^9}, 3.670127674801221*^9, 
   3.670210524237176*^9, 3.670235515356724*^9, 3.6705836702671747`*^9, 
   3.670583709882037*^9, 3.670586240668898*^9, 3.670588505034717*^9, 
   3.676007755921582*^9, 3.677301649072979*^9, 3.695027943691062*^9, 
   3.718447547297246*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PartialDeriv", "\[LeftDoubleBracket]", 
  RowBox[{"2", "NN"}], "\[RightDoubleBracket]"}]], "Input",
 CellChangeTimes->{{3.670123894302413*^9, 3.670123896716876*^9}, 
   3.670127578007668*^9}],

Cell[BoxData[
 FractionBox[
  RowBox[{"8", "-", 
   RowBox[{"8", " ", "we"}], "+", 
   RowBox[{"5", " ", "wi"}]}], 
  RowBox[{"8", "-", 
   RowBox[{"8", " ", "we"}], "+", 
   RowBox[{"8", " ", "wi"}]}]]], "Output",
 CellChangeTimes->{
  3.6701238971092377`*^9, 3.6701242026572723`*^9, 3.670124663758329*^9, {
   3.670124949232483*^9, 3.670124954730977*^9}, 3.670127674848015*^9, 
   3.6702105242815027`*^9, 3.67023551540746*^9, 3.670583670298376*^9, 
   3.670583709912869*^9, 3.670586240709857*^9, 3.6705885050966473`*^9, 
   3.676007755965081*^9, 3.677301649131103*^9, 3.6950279437428503`*^9, 
   3.7184475473438396`*^9}]
}, Open  ]],

Cell["\<\
Define a function that returns the analytical estimate for the partial \
derivative of a stimulation inhibitory neuron\
\>", "Text",
 CellChangeTimes->{{3.7184479067883883`*^9, 3.718447922492825*^9}, {
  3.718447963202877*^9, 3.718447967290813*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"derivPartialStim", "[", 
   RowBox[{"numStim_", ",", "NN_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"NN", "-", 
     RowBox[{"we", " ", "NN"}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"NN", "-", "numStim"}], ")"}], "wi"}]}], ")"}], "/", 
   RowBox[{"(", 
    RowBox[{"NN", "-", 
     RowBox[{"we", " ", "NN"}], "+", 
     RowBox[{"wi", " ", "NN"}]}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.670124078748921*^9, 3.6701241558571253`*^9}, {
  3.6701246288259897`*^9, 3.670124635200306*^9}, {3.6701275823442183`*^9, 
  3.670127590855233*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"derivPartialStim", "[", 
   RowBox[{"3", ",", "NN"}], "]"}], "//", "FullSimplify"}]], "Input",
 CellChangeTimes->{{3.670124135482319*^9, 3.6701241784083643`*^9}, {
   3.6701242139919863`*^9, 3.670124234983131*^9}, 3.6701246388916187`*^9, {
   3.670124669687181*^9, 3.670124671966599*^9}, {3.6701249469088593`*^9, 
   3.670124968172261*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{"8", "-", 
   RowBox[{"8", " ", "we"}], "+", 
   RowBox[{"5", " ", "wi"}]}], 
  RowBox[{"8", "-", 
   RowBox[{"8", " ", "we"}], "+", 
   RowBox[{"8", " ", "wi"}]}]]], "Output",
 CellChangeTimes->{{3.670124139866496*^9, 3.67012423528011*^9}, {
   3.670124639482113*^9, 3.670124672439417*^9}, {3.670124949269183*^9, 
   3.670124968504147*^9}, 3.670127674913156*^9, 3.670210524464189*^9, 
   3.6702355154625187`*^9, 3.670583670351775*^9, 3.67058370997001*^9, 
   3.67058624077182*^9, 3.670588505169585*^9, 3.676007756027076*^9, 
   3.677301649264538*^9, 3.695027943976226*^9, 3.718447547421198*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"1", "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"3", "wi"}], ")"}], "/", 
     RowBox[{"(", 
      RowBox[{"NN", " ", 
       SubscriptBox["\[Lambda]", "1"]}], ")"}]}], "-", 
    RowBox[{"derivPartialStim", "[", 
     RowBox[{"3", ",", "NN"}], "]"}]}], "/.", 
   RowBox[{"Reverse", "[", 
    RowBox[{
    "eigValRep", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
    "]"}]}], "//", "FullSimplify"}]], "Input",
 CellChangeTimes->{{3.6773032920235577`*^9, 3.677303378475041*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{{3.677303347415351*^9, 3.677303378659238*^9}, 
   3.695027944175931*^9, 3.7184475474662027`*^9}]
}, Open  ]],

Cell["\<\
Define a function that returns the analytical estimate for the partial \
derivative of a non-stimulated neuron\
\>", "Text",
 CellChangeTimes->{{3.718447936373451*^9, 3.718447961683097*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"derivPartialNonStim", "[", 
   RowBox[{"numStim_", ",", "NN_"}], "]"}], ":=", 
  RowBox[{"numStim", " ", 
   RowBox[{"wi", "/", 
    RowBox[{"(", 
     RowBox[{"NN", 
      RowBox[{"(", 
       RowBox[{"we", "-", "1", "-", "wi"}], ")"}]}], ")"}]}]}]}]], "Input",
 CellChangeTimes->{{3.670124642337442*^9, 3.670124656839911*^9}, {
  3.6701247747973633`*^9, 3.6701247927881804`*^9}, {3.670124831144867*^9, 
  3.670124841784265*^9}, {3.670124888416695*^9, 3.670124899174047*^9}, {
  3.6701275984317017`*^9, 3.6701275985431557`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"derivPartialNonStim", "[", 
   RowBox[{"3", ",", "NN"}], "]"}], "//", "FullSimplify"}]], "Input",
 CellChangeTimes->{{3.670124795732027*^9, 3.6701248162131357`*^9}, 
   3.670124848943623*^9, {3.670124904662064*^9, 3.670124910053464*^9}, {
   3.670124940261078*^9, 3.670124966172402*^9}}],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox[
   RowBox[{"3", " ", "wi"}], 
   RowBox[{"8", "-", 
    RowBox[{"8", " ", "we"}], "+", 
    RowBox[{"8", " ", "wi"}]}]]}]], "Output",
 CellChangeTimes->{{3.670124812015169*^9, 3.670124849112956*^9}, {
   3.670124901275194*^9, 3.6701249104324303`*^9}, {3.6701249425400257`*^9, 
   3.670124966490945*^9}, 3.670127674966066*^9, 3.670210524514523*^9, 
   3.670235515509153*^9, 3.670583670399828*^9, 3.670583710052537*^9, 
   3.670586240826071*^9, 3.670588505258185*^9, 3.676007756081616*^9, 
   3.67730164946496*^9, 3.69502794449013*^9, 3.7184475475377398`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"3", "wi"}], ")"}], "/", 
     RowBox[{"(", 
      RowBox[{"NN", " ", 
       SubscriptBox["\[Lambda]", "1"]}], ")"}]}], "-", 
    RowBox[{"derivPartialNonStim", "[", 
     RowBox[{"3", ",", "NN"}], "]"}]}], "/.", 
   RowBox[{"Reverse", "[", 
    RowBox[{
    "eigValRep", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
    "]"}]}], "//", "FullSimplify"}]], "Input",
 CellChangeTimes->{{3.6773034449225273`*^9, 3.6773034692153893`*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{{3.677303455471059*^9, 3.677303469505538*^9}, 
   3.69502794454362*^9, 3.718447547593005*^9}]
}, Open  ]],

Cell[TextData[{
 "Solve for the proportion of the inhibitory population ",
 StyleBox["p", "Input"],
 " that must be perturbed to provoke a paradoxical response"
}], "Text",
 CellChangeTimes->{{3.718447979851314*^9, 3.7184480068418417`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Reduce", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"derivPartialStim", "[", 
        RowBox[{"p", ",", "Num"}], "]"}], "<", "0"}], ",", 
      RowBox[{"we", ">", "0"}], ",", 
      RowBox[{"wi", ">", "0"}], ",", 
      RowBox[{"Num", ">", "0"}], ",", 
      RowBox[{"p", ">", "0"}], ",", 
      RowBox[{"p", "\[LessEqual]", " ", "Num"}], ",", "StabilityCond"}], 
     "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"Num", ",", "p"}], "}"}], ",", "Reals"}], "]"}], "//", 
  "FullSimplify"}]], "Input",
 CellChangeTimes->{{3.670125116554535*^9, 3.670125237322526*^9}, {
  3.670125267665708*^9, 3.670125269537635*^9}, {3.670127004884797*^9, 
  3.6701270191641083`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"1", "<", "we", "<", 
   RowBox[{"1", "+", "wi"}]}], "&&", 
  RowBox[{"\[Tau]", ">", "0"}], "&&", 
  RowBox[{
   FractionBox[
    RowBox[{"Num", " ", 
     RowBox[{"(", 
      RowBox[{"1", "-", "we", "+", "wi"}], ")"}]}], "wi"], "<", "p", 
   "\[LessEqual]", "Num"}]}]], "Output",
 CellChangeTimes->{{3.6701251270284643`*^9, 3.670125237626369*^9}, 
   3.670125269986258*^9, {3.670127011305664*^9, 3.670127019383751*^9}, 
   3.6701276751033297`*^9, 3.670210524628573*^9, 3.670235515555932*^9, 
   3.67058367052205*^9, 3.670583710098559*^9, 3.6705862409419603`*^9, 
   3.670588505383265*^9, 3.676007756214447*^9, 3.677301649598777*^9, 
   3.695027944634918*^9, 3.718447547677775*^9}]
}, Open  ]],

Cell[TextData[{
 "Define a function that analytically determines the minimum proportion of \
inhibitory neurons that must be perturbed to elicit a paradoxical response, \
for a given ",
 StyleBox["we", "Input"],
 " and ",
 StyleBox["wi", "Input"]
}], "Text",
 CellChangeTimes->{{3.718447785522551*^9, 3.7184478193128443`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"propMin", "[", 
   RowBox[{"Num_", ",", "we_", ",", "wi_"}], "]"}], ":=", 
  FractionBox[
   RowBox[{"Num", " ", 
    RowBox[{"(", 
     RowBox[{"1", "-", "we", "+", "wi"}], ")"}]}], 
   RowBox[{" ", "wi"}]]}]], "Input",
 CellChangeTimes->{{3.670126842765298*^9, 3.670126860128934*^9}, 
   3.670126982391094*^9}],

Cell["\<\
Solve for the parameters that permit a paradoxical response when only a \
single inhibitory neuron is perturbed\
\>", "Text",
 CellChangeTimes->{{3.718448015673923*^9, 3.71844805190073*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Reduce", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"derivPartialStim", "[", 
        RowBox[{"1", ",", "Num"}], "]"}], "<", "0"}], ",", 
      RowBox[{"we", ">", "0"}], ",", 
      RowBox[{"wi", ">", "0"}], ",", 
      RowBox[{"Num", ">", "0"}], ",", "StabilityCond"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"Num", ",", "wi", ",", "wi"}], "}"}], ",", "Reals"}], "]"}], "//",
   "FullSimplify"}]], "Input",
 CellChangeTimes->{{3.670125116554535*^9, 3.670125237322526*^9}, {
  3.670125267665708*^9, 3.670125269537635*^9}, {3.670127004884797*^9, 
  3.6701270191641083`*^9}, {3.670127423835498*^9, 3.6701274305393333`*^9}, {
  3.670127700013804*^9, 3.670127716155303*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Tau]", ">", "0"}], "&&", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"we", ">", "0"}], "&&", 
      RowBox[{"Num", ">", "0"}], "&&", 
      RowBox[{"we", "\[LessEqual]", "1"}], "&&", 
      RowBox[{"Num", "<", "1"}], "&&", 
      RowBox[{
       RowBox[{"Num", " ", 
        RowBox[{"(", 
         RowBox[{"1", "-", "we", "+", "wi"}], ")"}]}], "<", "wi"}]}], ")"}], "||", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"we", ">", "1"}], "&&", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"0", "<", "Num", "\[LessEqual]", "1"}], "&&", 
          RowBox[{
           RowBox[{"1", "+", "wi"}], ">", "we"}]}], ")"}], "||", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Num", ">", "1"}], "&&", 
          RowBox[{"we", "<", 
           RowBox[{"1", "+", "wi"}]}], "&&", 
          RowBox[{
           RowBox[{
            RowBox[{"Num", " ", "we"}], "+", "wi"}], ">", 
           RowBox[{"Num", "+", 
            RowBox[{"Num", " ", "wi"}]}]}]}], ")"}]}], ")"}]}], ")"}]}], 
   ")"}]}]], "Output",
 CellChangeTimes->{{3.6701274242901506`*^9, 3.670127430833871*^9}, {
   3.67012767523492*^9, 3.67012771649378*^9}, 3.670210524865816*^9, 
   3.670235515620172*^9, 3.670583670720159*^9, 3.670583710170671*^9, 
   3.670586241176998*^9, 3.670588505596533*^9, 3.676007756421288*^9, 
   3.677301649807269*^9, 3.695027944834938*^9, 3.718447547895054*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"propMin", "[", 
  RowBox[{"100", ",", "5", ",", "20"}], "]"}]], "Input",
 CellChangeTimes->{{3.6702111996672907`*^9, 3.6702112191531677`*^9}, {
  3.6702112803185873`*^9, 3.6702112805661173`*^9}}],

Cell[BoxData["80"], "Output",
 CellChangeTimes->{{3.6702112032619877`*^9, 3.6702112194780483`*^9}, 
   3.670211280985263*^9, 3.670235515773666*^9, 3.6705836713130217`*^9, 
   3.670583710282167*^9, 3.67058624157778*^9, 3.670588505631624*^9, 
   3.67600775657952*^9, 3.677301649864402*^9, 3.695027944878682*^9, 
   3.718447548040839*^9}]
}, Open  ]],

Cell[TextData[{
 "Verify that the simplification for ",
 StyleBox["propMin", "Input"],
 " is valid"
}], "Text",
 CellChangeTimes->{{3.7184480704774446`*^9, 3.718448084715085*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "NN"}], " ", 
       SubscriptBox["\[Lambda]", "1"]}], ")"}], "/", "wi"}], "-", 
    RowBox[{"propMin", "[", 
     RowBox[{"NN", ",", "we", ",", "wi"}], "]"}]}], "/.", 
   RowBox[{"Reverse", "[", 
    RowBox[{
    "eigValRep", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
    "]"}]}], "//", "FullSimplify"}]], "Input",
 CellChangeTimes->{{3.6773035491951733`*^9, 3.677303616741922*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{{3.677303570379602*^9, 3.6773036170186863`*^9}, 
   3.695027944911385*^9, 3.718447548094697*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
WindowSize->{761, 751},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
FrontEndVersion->"9.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (January 25, \
2013)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[580, 22, 566, 12, 242, "Title"],
Cell[CellGroupData[{
Cell[1171, 38, 400, 9, 120, "Section"],
Cell[1574, 49, 4043, 98, 131, "Input"],
Cell[5620, 149, 1231, 29, 46, "Input"],
Cell[6854, 180, 138, 3, 30, "Text"],
Cell[CellGroupData[{
Cell[7017, 187, 1189, 23, 97, "Input"],
Cell[8209, 212, 345, 7, 48, "Output"],
Cell[8557, 221, 332, 7, 48, "Output"],
Cell[8892, 230, 344, 7, 48, "Output"]
}, Open  ]],
Cell[9251, 240, 131, 1, 30, "Text"],
Cell[9385, 243, 679, 16, 28, "Input"],
Cell[CellGroupData[{
Cell[10089, 263, 35, 0, 28, "Input"],
Cell[10127, 265, 550, 14, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[10726, 285, 342, 8, 100, "Section"],
Cell[CellGroupData[{
Cell[11093, 297, 941, 22, 63, "Input"],
Cell[12037, 321, 1816, 33, 28, "Output"]
}, Open  ]],
Cell[13868, 357, 217, 6, 31, "Text"],
Cell[CellGroupData[{
Cell[14110, 367, 293, 6, 28, "Input"],
Cell[14406, 375, 608, 11, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[15051, 391, 328, 9, 28, "Input"],
Cell[15382, 402, 602, 11, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[16033, 419, 352, 8, 100, "Section"],
Cell[16388, 429, 841, 16, 28, "Input"],
Cell[17232, 447, 894, 21, 28, "Input"],
Cell[CellGroupData[{
Cell[18151, 472, 558, 11, 46, "Input"],
Cell[18712, 485, 1558, 42, 68, "Output"],
Cell[20273, 529, 1585, 43, 68, "Output"]
}, Open  ]],
Cell[21873, 575, 598, 19, 34, "Text"],
Cell[22474, 596, 612, 18, 36, "Input"],
Cell[23089, 616, 1183, 26, 37, "Input"],
Cell[CellGroupData[{
Cell[24297, 646, 1368, 39, 82, "Input"],
Cell[25668, 687, 495, 13, 50, "Output"],
Cell[26166, 702, 251, 5, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[26454, 712, 1544, 43, 84, "Input"],
Cell[28001, 757, 572, 14, 50, "Output"],
Cell[28576, 773, 326, 6, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[28951, 785, 545, 9, 172, "Section"],
Cell[CellGroupData[{
Cell[29521, 798, 134, 1, 44, "Subsection"],
Cell[29658, 801, 925, 23, 46, "Input"],
Cell[CellGroupData[{
Cell[30608, 828, 763, 14, 28, "Input"],
Cell[31374, 844, 331, 8, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[31742, 857, 759, 15, 28, "Input"],
Cell[32504, 874, 1396, 22, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[33937, 901, 239, 4, 28, "Input"],
Cell[34179, 907, 622, 13, 48, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[34838, 925, 375, 7, 28, "Input"],
Cell[35216, 934, 626, 12, 50, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[35891, 952, 130, 1, 44, "Subsection"],
Cell[36024, 955, 488, 12, 28, "Input"],
Cell[CellGroupData[{
Cell[36537, 971, 928, 17, 28, "Input"],
Cell[37468, 990, 1460, 25, 48, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[38965, 1020, 349, 7, 28, "Input"],
Cell[39317, 1029, 739, 15, 48, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[40093, 1049, 547, 16, 28, "Input"],
Cell[40643, 1067, 329, 9, 48, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[41021, 1082, 135, 1, 44, "Subsection"],
Cell[41159, 1085, 796, 21, 52, "Input"],
Cell[CellGroupData[{
Cell[41980, 1110, 463, 11, 28, "Input"],
Cell[42446, 1123, 391, 5, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[42874, 1133, 193, 3, 28, "Input"],
Cell[43070, 1138, 322, 6, 48, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[43441, 1150, 123, 1, 44, "Subsection"],
Cell[43567, 1153, 1368, 34, 80, "Input"],
Cell[CellGroupData[{
Cell[44960, 1191, 656, 18, 46, "Input"],
Cell[45619, 1211, 655, 18, 45, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[46311, 1234, 1129, 24, 46, "Input"],
Cell[47443, 1260, 557, 8, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[48037, 1273, 805, 20, 80, "Input"],
Cell[48845, 1295, 484, 13, 48, "Output"],
Cell[49332, 1310, 475, 12, 45, "Output"],
Cell[49810, 1324, 504, 14, 50, "Output"],
Cell[50317, 1340, 524, 15, 50, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[50878, 1360, 708, 15, 80, "Input"],
Cell[51589, 1377, 549, 17, 51, "Output"],
Cell[52141, 1396, 551, 17, 51, "Output"],
Cell[52695, 1415, 504, 15, 52, "Output"],
Cell[53202, 1432, 504, 15, 52, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[53743, 1452, 2035, 59, 98, "Input"],
Cell[55781, 1513, 448, 9, 47, "Output"],
Cell[56232, 1524, 448, 9, 47, "Output"],
Cell[56683, 1535, 463, 10, 49, "Output"],
Cell[57149, 1547, 461, 10, 49, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[57647, 1562, 1070, 30, 46, "Input"],
Cell[58720, 1594, 437, 9, 45, "Output"],
Cell[59160, 1605, 365, 6, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[59586, 1618, 563, 12, 135, "Section"],
Cell[CellGroupData[{
Cell[60174, 1634, 173, 3, 44, "Subsection"],
Cell[60350, 1639, 597, 14, 28, "Input"],
Cell[CellGroupData[{
Cell[60972, 1657, 931, 22, 63, "Input"],
Cell[61906, 1681, 6297, 111, 184, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[68252, 1798, 370, 8, 88, "Subsection"],
Cell[CellGroupData[{
Cell[68647, 1810, 1344, 29, 46, "Input"],
Cell[69994, 1841, 3575, 111, 162, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[73606, 1957, 215, 4, 28, "Input"],
Cell[73824, 1963, 622, 14, 48, "Output"]
}, Open  ]],
Cell[74461, 1980, 259, 5, 49, "Text"],
Cell[74723, 1987, 607, 17, 28, "Input"],
Cell[CellGroupData[{
Cell[75355, 2008, 375, 7, 28, "Input"],
Cell[75733, 2017, 633, 13, 48, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[76403, 2035, 544, 16, 46, "Input"],
Cell[76950, 2053, 144, 2, 28, "Output"]
}, Open  ]],
Cell[77109, 2058, 200, 4, 49, "Text"],
Cell[77312, 2064, 564, 13, 28, "Input"],
Cell[CellGroupData[{
Cell[77901, 2081, 322, 6, 28, "Input"],
Cell[78226, 2089, 601, 12, 48, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[78864, 2106, 539, 16, 46, "Input"],
Cell[79406, 2124, 141, 2, 28, "Output"]
}, Open  ]],
Cell[79562, 2129, 240, 5, 50, "Text"],
Cell[CellGroupData[{
Cell[79827, 2138, 736, 20, 46, "Input"],
Cell[80566, 2160, 715, 16, 48, "Output"]
}, Open  ]],
Cell[81296, 2179, 326, 8, 50, "Text"],
Cell[81625, 2189, 347, 10, 49, "Input"],
Cell[81975, 2201, 200, 4, 49, "Text"],
Cell[CellGroupData[{
Cell[82200, 2209, 747, 18, 46, "Input"],
Cell[82950, 2229, 1472, 40, 46, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[84459, 2274, 219, 4, 28, "Input"],
Cell[84681, 2280, 335, 5, 28, "Output"]
}, Open  ]],
Cell[85031, 2288, 179, 5, 31, "Text"],
Cell[CellGroupData[{
Cell[85235, 2297, 512, 15, 28, "Input"],
Cell[85750, 2314, 144, 2, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

